
import os
from time import sleep

import gdbremote_testcase
import lldbgdbserverutils
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestGdbRemoteAttachWait(gdbremote_testcase.GdbRemoteTestCaseBase):

    def _set_up_inferior(self):
        self._exe_to_attach = '%s_%d' % (self.testMethodName, os.getpid())
        self.build(dictionary={'EXE': self._exe_to_attach, 'CXX_SOURCES': 'main.cpp'})

        if self.getPlatform() != "windows":
            # Use a shim to ensure that the process is ready to be attached from
            # the get-go.
            self._exe_to_run = "shim"
            self._run_args = [self.getBuildArtifact(self._exe_to_attach)]
            self.build(dictionary={'EXE': self._exe_to_run, 'CXX_SOURCES':
                'shim.cpp'})
        else:
            self._exe_to_run = self._exe_to_attach
            self._run_args = []

    def _launch_inferior(self, args):
        inferior = self.spawnSubprocess(self.getBuildArtifact(self._exe_to_run),
                args)
        self.assertIsNotNone(inferior)
        self.assertTrue(inferior.pid > 0)
        self.assertTrue(
            lldbgdbserverutils.process_is_running(
                inferior.pid, True))
        return inferior

    def _launch_and_wait_for_init(self):
        sync_file_path = lldbutil.append_to_process_working_directory(self,
                "process_ready")
        inferior = self._launch_inferior(self._run_args + [sync_file_path])
        lldbutil.wait_for_file_on_target(self, sync_file_path)
        return inferior

    def _attach_packet(self, packet_type):
        return "read packet: ${};{}#00".format(packet_type,
                lldbgdbserverutils.gdbremote_hex_encode_string(self._exe_to_attach))


    @skipIfWindows # This test is flaky on Windows
    def test_attach_with_vAttachWait(self):
        self._set_up_inferior()

        self.set_inferior_startup_attach_manually()
        server = self.connect_to_debug_monitor()
        self.do_handshake()

        # Launch the first inferior (we shouldn't attach to this one).
        self._launch_and_wait_for_init()

        self.test_sequence.add_log_lines([self._attach_packet("vAttachWait")],
                True)
        # Run the stream until attachWait.
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Sleep so we're sure that the inferior is launched after we ask for the attach.
        sleep(1)

        # Launch the second inferior (we SHOULD attach to this one).
        inferior_to_attach = self._launch_inferior(self._run_args)

        # Make sure the attach succeeded.
        self.test_sequence.add_log_lines([
            {"direction": "send",
             "regex": r"^\$T([0-9a-fA-F]{2})[^#]*#[0-9a-fA-F]{2}$",
             "capture": {1: "stop_signal_hex"}},
        ], True)
        self.add_process_info_collection_packets()


        # Run the stream sending the response..
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Gather process info response.
        process_info = self.parse_process_info_response(context)
        self.assertIsNotNone(process_info)

        # Ensure the process id matches what we expected.
        pid_text = process_info.get('pid', None)
        self.assertIsNotNone(pid_text)
        reported_pid = int(pid_text, base=16)
        self.assertEqual(reported_pid, inferior_to_attach.pid)

    @skipIfWindows # This test is flaky on Windows
    def test_launch_before_attach_with_vAttachOrWait(self):
        self._set_up_inferior()

        self.set_inferior_startup_attach_manually()
        server = self.connect_to_debug_monitor()
        self.do_handshake()

        inferior = self._launch_and_wait_for_init()

        # Add attach packets.
        self.test_sequence.add_log_lines([
            # Do the attach.
            self._attach_packet("vAttachOrWait"),
            # Expect a stop notification from the attach.
            {"direction": "send",
             "regex": r"^\$T([0-9a-fA-F]{2})[^#]*#[0-9a-fA-F]{2}$",
             "capture": {1: "stop_signal_hex"}},
        ], True)
        self.add_process_info_collection_packets()

        # Run the stream
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Gather process info response
        process_info = self.parse_process_info_response(context)
        self.assertIsNotNone(process_info)

        # Ensure the process id matches what we expected.
        pid_text = process_info.get('pid', None)
        self.assertIsNotNone(pid_text)
        reported_pid = int(pid_text, base=16)
        self.assertEqual(reported_pid, inferior.pid)

    @skipIfWindows # This test is flaky on Windows
    def test_launch_after_attach_with_vAttachOrWait(self):
        self._set_up_inferior()

        self.set_inferior_startup_attach_manually()
        server = self.connect_to_debug_monitor()
        self.do_handshake()

        self.test_sequence.add_log_lines([self._attach_packet("vAttachOrWait")],
                True)
        # Run the stream until attachWait.
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Sleep so we're sure that the inferior is launched after we ask for the attach.
        sleep(1)

        # Launch the inferior.
        inferior = self._launch_inferior(self._run_args)

        # Make sure the attach succeeded.
        self.test_sequence.add_log_lines([
            {"direction": "send",
             "regex": r"^\$T([0-9a-fA-F]{2})[^#]*#[0-9a-fA-F]{2}$",
             "capture": {1: "stop_signal_hex"}},
        ], True)
        self.add_process_info_collection_packets()


        # Run the stream sending the response..
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Gather process info response.
        process_info = self.parse_process_info_response(context)
        self.assertIsNotNone(process_info)

        # Ensure the process id matches what we expected.
        pid_text = process_info.get('pid', None)
        self.assertIsNotNone(pid_text)
        reported_pid = int(pid_text, base=16)
        self.assertEqual(reported_pid, inferior.pid)



#!/usr/bin/env python3
import argparse
import os
import sys

BASE_PATH = os.path.dirname(os.path.abspath(__file__))
HTML_TEMPLATE_NAME = 'd3-graphviz-template.html'
HTML_TEMPLATE_PATH = os.path.join(BASE_PATH, HTML_TEMPLATE_NAME)

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('dotfile', nargs='?', type=argparse.FileType('r'),
                        default=sys.stdin,
                        help='Input .dot file, reads from stdin if not set')
    parser.add_argument('htmlfile', nargs='?', type=argparse.FileType('w'),
                        default=sys.stdout,
                        help='Output .html file, writes to stdout if not set')
    args = parser.parse_args()

    template = open(HTML_TEMPLATE_PATH, 'r')

    for line in template:
        if "<INSERT_DOT>" in line:
            print(args.dotfile.read(), file=args.htmlfile, end='')
        else:
            print(line, file=args.htmlfile, end='')

if __name__ == "__main__":
    main()



"""
Test we can understand various layouts of the libc++'s std::string
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import functools


class LibcxxStringDataFormatterSimulatorTestCase(TestBase):

    NO_DEBUG_INFO_TESTCASE = True

    def _run_test(self, defines):
        cxxflags_extras = " ".join(["-D%s" % d for d in defines])
        self.build(dictionary=dict(CXXFLAGS_EXTRAS=cxxflags_extras))
        lldbutil.run_to_source_breakpoint(self, '// Break here',
                lldb.SBFileSpec("main.cpp"))
        self.expect_var_path("shortstring", summary='"short"')
        self.expect_var_path("longstring", summary='"I am a very long string"')

for v in [None, "ALTERNATE_LAYOUT"]:
    for r in range(5):
        name = "test_r%d"%r
        defines = ["REVISION=%d"%r]
        if v:
            name += "_" + v
            defines += [v]
        f = functools.partialmethod(
                LibcxxStringDataFormatterSimulatorTestCase._run_test, defines)
        setattr(LibcxxStringDataFormatterSimulatorTestCase, name, f)



import lldb
from intelpt_testcase import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *

class TestTraceDumpInfo(TraceIntelPTTestCaseBase):

    def testErrorMessages(self):
        # We first check the output when there are no targets
        self.expect("thread trace dump info",
            substrs=["error: invalid target, create a target using the 'target create' command"],
            error=True)

        # We now check the output when there's a non-running target
        self.expect("target create " +
            os.path.join(self.getSourceDir(), "intelpt-trace", "a.out"))

        self.expect("thread trace dump info",
            substrs=["error: Command requires a current process."],
            error=True)

        # Now we check the output when there's a running target without a trace
        self.expect("b main")
        self.expect("run")

        self.expect("thread trace dump info",
            substrs=["error: Process is not being traced"],
            error=True)

    def testDumpRawTraceSize(self):
        self.expect("trace load -v " +
        os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json"),
        substrs=["intel-pt"])

        self.expect("thread trace dump info",
            substrs=['''thread #1: tid = 3842849

  Trace technology: intel-pt

  Total number of trace items: 23

  Memory usage:
    Raw trace size: 4 KiB
    Total approximate memory usage (excluding raw trace): 0.20 KiB
    Average memory usage per item (excluding raw trace): 9.00 bytes

  Timing for this thread:
    Decoding instructions: ''', '''

  Events:
    Number of individual events: 2
      software disabled tracing: 2

  Errors:
    Number of TSC decoding errors: 0'''],
            patterns=["Decoding instructions: \d.\d\ds"])

    def testDumpRawTraceSizeJSON(self):
        self.expect("trace load -v " +
        os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json"),
        substrs=["intel-pt"])

        self.expect("thread trace dump info --json ",
            substrs=['''{
  "traceTechnology": "intel-pt",
  "threadStats": {
    "tid": 3842849,
    "traceItemsCount": 23,
    "memoryUsage": {
      "totalInBytes": "207",
      "avgPerItemInBytes": 9
    },
    "timingInSeconds": {
      "Decoding instructions": 0''', '''
    },
    "events": {
      "totalCount": 2,
      "individualCounts": {
        "software disabled tracing": 2
      }
    },
    "errorItems": {
      "total": 0,
      "individualErrors": {}
    }
  },
  "globalStats": {
    "timingInSeconds": {}
  }
}'''])



# RUN: SUPPORTLIB=%mlir_runner_utils_dir/libmlir_c_runner_utils%shlibext %PYTHON %s | FileCheck %s

import numpy as np
import os
import sys

_SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(_SCRIPT_PATH)
from tools import mlir_pytaco_api as pt

compressed = pt.compressed
dense = pt.dense

passed = 0
all_types = [
    pt.int8, pt.int16, pt.int32, pt.int64, pt.float16, pt.float32, pt.float64
]
for t in all_types:
  i, j = pt.get_index_vars(2)
  A = pt.tensor([2, 3], dtype=t)
  B = pt.tensor([2, 3], dtype=t)
  C = pt.tensor([2, 3], compressed, dtype=t)
  A.insert([0, 1], 10)
  A.insert([1, 2], 40)
  B.insert([0, 0], 20)
  B.insert([1, 2], 30)
  C[i, j] = A[i, j] + B[i, j]

  indices, values = C.get_coordinates_and_values()
  passed += isinstance(values[0], t.value)
  passed += np.array_equal(indices, [[0, 0], [0, 1], [1, 2]])
  passed += np.allclose(values, [20.0, 10.0, 70.0])

# CHECK: Number of passed: 21
print("Number of passed:", passed)



# RUN: %PYTHON %s | FileCheck %s
# This is just a smoke test that the dialect is functional.

from mlir.ir import *
from mlir.dialects import ml_program


def constructAndPrintInModule(f):
  print("\nTEST:", f.__name__)
  with Context(), Location.unknown():
    module = Module.create()
    with InsertionPoint(module.body):
      f()
    print(module)
  return f


# CHECK-LABEL: testFuncOp
@constructAndPrintInModule
def testFuncOp():
  # CHECK: ml_program.func @foobar(%arg0: si32) -> si32
  f = ml_program.FuncOp(
      name="foobar",
      type=([IntegerType.get_signed(32)], [IntegerType.get_signed(32)]))
  block = f.add_entry_block()
  with InsertionPoint(block):
    # CHECK: ml_program.return
    ml_program.ReturnOp([block.arguments[0]])



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

# This file contains the utilities to support testing.

import numpy as np


def compare_sparse_tns(expected: str, actual: str, rtol: float = 0.0001) -> bool:
  """Compares sparse tensor actual output file with expected output file.

  This routine assumes the input files are in FROSTT format. See
  http://frostt.io/tensors/file-formats.html for FROSTT (.tns) format.

  It also assumes the first line in the output file is a comment line.

  """
  with open(actual, "r") as actual_f:
    with open(expected, "r") as expected_f:
      # Skip the first comment line.
      _ = actual_f.readline()
      _ = expected_f.readline()

      # Compare the two lines of meta data
      if (actual_f.readline() != expected_f.readline() or
          actual_f.readline() != expected_f.readline()):
        return FALSE

  actual_data = np.loadtxt(actual, np.float64, skiprows=3)
  expected_data = np.loadtxt(expected, np.float64, skiprows=3)
  return np.allclose(actual_data, expected_data, rtol=rtol)


def file_as_string(file: str) -> str:
  """Returns contents of file as string."""
  with open(file, "r") as f:
    return f.read()


def run_test(f):
  """Prints the test name and runs the test."""
  print(f.__name__)
  f()
  return f



"""
Test lldb data formatter subsystem for std::span
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class LibcxxSpanDataFormatterTestCase(TestBase):

    def findVariable(self, name):
        var = self.frame().FindVariable(name)
        self.assertTrue(var.IsValid())
        return var

    def check_size(self, var_name, size):
        var = self.findVariable(var_name)
        self.assertEqual(var.GetNumChildren(), size)

    def check_numbers(self, var_name):
        """Helper to check that data formatter sees contents of std::span correctly"""

        expectedSize = 5
        self.check_size(var_name, expectedSize)

        self.expect_expr(
                var_name, result_type=f'std::span<int, {expectedSize}>',
                result_summary=f'size={expectedSize}',
                result_children=[
                    ValueCheck(name='[0]', value='1'),
                    ValueCheck(name='[1]', value='12'),
                    ValueCheck(name='[2]', value='123'),
                    ValueCheck(name='[3]', value='1234'),
                    ValueCheck(name='[4]', value='12345')
        ])

        # check access-by-index
        self.expect_var_path(f'{var_name}[0]', type='int', value='1')
        self.expect_var_path(f'{var_name}[1]', type='int', value='12')
        self.expect_var_path(f'{var_name}[2]', type='int', value='123')
        self.expect_var_path(f'{var_name}[3]', type='int', value='1234')
        self.expect_var_path(f'{var_name}[4]', type='int', value='12345')

    @add_test_categories(['libc++'])
    @skipIf(compiler='clang', compiler_version=['<', '11.0'])
    def test_with_run_command(self):
        """Test that std::span variables are formatted correctly when printed."""
        self.build()
        (self.target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.cpp', False))

        lldbutil.continue_to_breakpoint(process, bkpt)

        # std::span of std::array with extents known at compile-time
        self.check_numbers('numbers_span')

        # check access to synthetic children for static spans
        self.runCmd('type summary add --summary-string "item 0 is ${var[0]}" -x "std::span<" span')
        self.expect_expr('numbers_span',
                         result_type='std::span<int, 5>',
                         result_summary='item 0 is 1')

        self.runCmd('type summary add --summary-string "item 0 is ${svar[0]}" -x "std::span<" span')
        self.expect_expr('numbers_span',
                         result_type='std::span<int, 5>',
                         result_summary='item 0 is 1')

        self.runCmd('type summary delete span')

        # New span with strings
        lldbutil.continue_to_breakpoint(process, bkpt)

        expectedStringSpanChildren = [ ValueCheck(name='[0]', summary='"smart"'),
                                       ValueCheck(name='[1]', summary='"!!!"') ]

        self.expect_var_path('strings_span',
                             summary='size=2',
                             children=expectedStringSpanChildren)

        # check access to synthetic children for dynamic spans
        self.runCmd('type summary add --summary-string "item 0 is ${var[0]}" dynamic_string_span')
        self.expect_var_path('strings_span', summary='item 0 is "smart"')

        self.runCmd('type summary add --summary-string "item 0 is ${svar[0]}" dynamic_string_span')
        self.expect_var_path('strings_span', summary='item 0 is "smart"')

        self.runCmd('type summary delete dynamic_string_span')

        # test summaries based on synthetic children
        self.runCmd(
                'type summary add --summary-string "span has ${svar%#} items" -e dynamic_string_span')

        self.expect_var_path('strings_span', summary='span has 2 items')

        self.expect_var_path('strings_span',
                             summary='span has 2 items',
                             children=expectedStringSpanChildren)

        # check access-by-index
        self.expect_var_path('strings_span[0]', summary='"smart"');
        self.expect_var_path('strings_span[1]', summary='"!!!"');

        # Newly inserted value not visible to span
        lldbutil.continue_to_breakpoint(process, bkpt)

        self.expect_expr('strings_span',
                         result_summary='span has 2 items',
                         result_children=expectedStringSpanChildren)

        self.runCmd('type summary delete dynamic_string_span')

        lldbutil.continue_to_breakpoint(process, bkpt)

        # Empty spans
        self.expect_expr('static_zero_span',
                         result_type='std::span<int, 0>',
                         result_summary='size=0')
        self.check_size('static_zero_span', 0)

        self.expect_expr('dynamic_zero_span',
                         result_summary='size=0')
        self.check_size('dynamic_zero_span', 0)

        # Nested spans
        self.expect_expr('nested',
                         result_summary='size=2',
                         result_children=[
                             ValueCheck(name='[0]', summary='size=2',
                                        children=expectedStringSpanChildren),
                             ValueCheck(name='[1]', summary='size=2',
                                        children=expectedStringSpanChildren)
        ])
        self.check_size('nested', 2)

    @add_test_categories(['libc++'])
    @skipIf(compiler='clang', compiler_version=['<', '11.0'])
    def test_ref_and_ptr(self):
        """Test that std::span is correctly formatted when passed by ref and ptr"""
        self.build()
        (self.target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, 'Stop here to check by ref', lldb.SBFileSpec('main.cpp', False))

        # The reference should display the same was as the value did
        self.check_numbers('ref')

        # The pointer should just show the right number of elements:

        ptrAddr = self.findVariable('ptr').GetValue()
        self.expect_expr('ptr', result_type='std::span<int, 5> *',
                         result_summary=f'{ptrAddr} size=5')



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects import linalg

from mlir.dialects.linalg.opdsl.lang import *

T1 = TV.T1
T2 = TV.T2


@linalg_structured_op
def fill_poly(value=ScalarDef(T1), O=TensorDef(U, output=True)):
  O[None] = TypeFn.cast_signed(U, value)

@linalg_structured_op
def fill_rank_zero_poly(I=TensorDef(T1), O=TensorDef(U, output=True)):
  O[None] = TypeFn.cast_signed(U, I[None])

with Context() as ctx, Location.unknown():
  module = Module.create()
  f32 = F32Type.get()
  with InsertionPoint(module.body):

    # Fill indexing maps.
    # CHECK-DAG: #[[$MAP0:.+]] = affine_map<() -> ()>
    # CHECK-DAG: #[[$MAP1:.+]] = affine_map<(d0, d1) -> ()>
    # CHECK-DAG: #[[$MAP2:.+]] = affine_map<(d0, d1) -> (d0, d1)>
    # CHECK-DAG: #[[$MAP3:.+]] = affine_map<(d0, d1, d2) -> ()>
    # CHECK-DAG: #[[$MAP4:.+]] = affine_map<(d0, d1, d2) -> (d0, d1, d2)>

    # CHECK-LABEL: @test_fill_0d
    # CHECK: linalg.generic
    # CHECK-SAME: indexing_maps = [#[[$MAP0]], #[[$MAP0]]
    # CHECK-SAME: iterator_types = []
    @func.FuncOp.from_py_func(f32, RankedTensorType.get([], f32))
    def test_fill_0d(value, init_result):
      return fill_poly(value, outs=[init_result])

    # CHECK-LABEL: @test_fill_2d
    # CHECK: linalg.generic
    # CHECK-SAME: indexing_maps = [#[[$MAP1]], #[[$MAP2]]]
    # CHECK-SAME: iterator_types = ["parallel", "parallel"]
    @func.FuncOp.from_py_func(f32, RankedTensorType.get([4, 16], f32))
    def test_fill_2d(value, init_result):
      return fill_poly(value, outs=[init_result])

    # CHECK-LABEL: @test_fill_rank_zero_3d
    # CHECK: linalg.generic
    # CHECK-SAME: indexing_maps = [#[[$MAP3]], #[[$MAP4]]]
    # CHECK-SAME: iterator_types = ["parallel", "parallel", "parallel"]
    @func.FuncOp.from_py_func(
        RankedTensorType.get([], f32), RankedTensorType.get([4, 8, 16], f32))
    def test_fill_rank_zero_3d(input, init_result):
      return fill_rank_zero_poly(input, outs=[init_result])

print(module)




import unittest2

from lldbsuite.test.decorators import *
from lldbsuite.test.concurrent_base import ConcurrentEventsBase
from lldbsuite.test.lldbtest import TestBase


@skipIfWindows
class ConcurrentDelayedCrashWithBreakpointWatchpoint(ConcurrentEventsBase):

    # Atomic sequences are not supported yet for MIPS in LLDB.
    @skipIf(triple='^mips')
    @add_test_categories(["watchpoint"])
    def test(self):
        """ Test a thread with a delayed crash while other threads hit a watchpoint and a breakpoint. """
        self.build()
        self.do_thread_actions(num_delay_crash_threads=1,
                               num_breakpoint_threads=1,
                               num_watchpoint_threads=1)



# RUN: SUPPORTLIB=%mlir_runner_utils_dir/libmlir_c_runner_utils%shlibext %PYTHON %s | FileCheck %s

import numpy as np
import os
import sys

_SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(_SCRIPT_PATH)
from tools import mlir_pytaco_api as pt

i, j = pt.get_index_vars(2)
# Both tensors are true dense tensors.
A = pt.from_array(np.full([2,3], 1, dtype=np.float64))
B = pt.from_array(np.full([2,3], 2, dtype=np.float64))
# Define the result tensor as a true dense tensor. The parameter is_dense=True
# is an MLIR-PyTACO extension.
C = pt.tensor([2, 3], dtype=pt.float64, is_dense=True)

C[i, j] = A[i, j] + B[i, j]

# CHECK: [3. 3. 3. 3. 3. 3.]
print(C.to_array().reshape(6))



# RUN: SUPPORTLIB=%mlir_runner_utils_dir/libmlir_c_runner_utils%shlibext %PYTHON %s | FileCheck %s

import filecmp
import numpy as np
import os
import sys
import tempfile

_SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(_SCRIPT_PATH)

from tools import mlir_pytaco_api as pt
from tools import testing_utils as utils

i, j, k, l, m = pt.get_index_vars(5)

# Set up scalar.
alpha = pt.tensor(42.0)

# Set up some sparse tensors with different dim annotations and ordering.
S = pt.tensor([8, 8, 8],
              pt.format([pt.compressed, pt.dense, pt.compressed], [1, 0, 2]))
X = pt.tensor([8, 8, 8],
              pt.format([pt.compressed, pt.compressed, pt.compressed],
                        [1, 0, 2]))
S.insert([0, 0, 0], 2.0)
S.insert([1, 1, 1], 3.0)
S.insert([4, 4, 4], 4.0)
S.insert([7, 7, 7], 5.0)

X[i, j, k] = alpha[0] * S[i, j, k]

# Set up tensors with a dense last dimension. This results in a full
# enveloping storage of all last "rows" with one or more nonzeros.
T = pt.tensor([1, 2, 3, 4, 5],
              pt.format([
                  pt.compressed, pt.compressed, pt.compressed, pt.compressed,
                  pt.dense
              ]))
Y = pt.tensor([1, 2, 3, 4, 5],
              pt.format([
                  pt.compressed, pt.compressed, pt.compressed, pt.compressed,
                  pt.dense
              ]))
T.insert([0, 1, 2, 3, 4], -2.0)

Y[i, j, k, l, m] = alpha[0] * T[i, j, k, l, m]

# Set up a sparse tensor and dense tensor with different access.
U = pt.tensor([2, 3], pt.format([pt.compressed, pt.compressed], [1, 0]))
Z = pt.tensor([3, 2], pt.format([pt.dense, pt.dense]))
U.insert([1, 2], 3.0)

Z[i, j] = alpha[0] * U[j, i]

x_expected = """; extended FROSTT format
3 4
8 8 8
1 1 1 84
2 2 2 126
5 5 5 168
8 8 8 210
"""

y_expected = """; extended FROSTT format
5 5
1 2 3 4 5
1 2 3 4 1 0
1 2 3 4 2 0
1 2 3 4 3 0
1 2 3 4 4 0
1 2 3 4 5 -84
"""

z_expected = """; extended FROSTT format
2 6
3 2
1 1 0
1 2 0
2 1 0
2 2 0
3 1 0
3 2 126
"""

# Force evaluation of the kernel by writing out X.
with tempfile.TemporaryDirectory() as test_dir:
  x_file = os.path.join(test_dir, 'X.tns')
  pt.write(x_file, X)
  y_file = os.path.join(test_dir, 'Y.tns')
  pt.write(y_file, Y)
  z_file = os.path.join(test_dir, 'Z.tns')
  pt.write(z_file, Z)
  #
  # CHECK: Compare result True True True
  #
  x_data = utils.file_as_string(x_file)
  y_data = utils.file_as_string(y_file)
  z_data = utils.file_as_string(z_file)
  print(
      f'Compare result {x_data == x_expected} {y_data == y_expected} {z_data == z_expected}'
  )




import unittest2
import os
import shutil

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestClangModuleHashMismatch(TestBase):

    @skipIf(debug_info=no_match(["gmodules"]))
    def test_expr(self):
        with open(self.getBuildArtifact("module.modulemap"), "w") as f:
            f.write("""
                    module Foo { header "f.h" }
                    """)
        with open(self.getBuildArtifact("f.h"), "w") as f:
            f.write("""
                    typedef int my_int;
                    void f() {}
                    """)

        mod_cache = self.getBuildArtifact("private-module-cache")
        if os.path.isdir(mod_cache):
          shutil.rmtree(mod_cache)
        self.build()
        self.assertTrue(os.path.isdir(mod_cache), "module cache exists")

        logfile = self.getBuildArtifact("host.log")
        with open(logfile, 'w') as f:
            sbf = lldb.SBFile(f.fileno(), 'w', False)
            status = self.dbg.SetErrorFile(sbf)
            self.assertSuccess(status)

            target, _, _, _ = lldbutil.run_to_source_breakpoint(
                self, "break here", lldb.SBFileSpec("main.m"))
            target.GetModuleAtIndex(0).FindTypes('my_int')

        with open(logfile, 'r') as f:
            for line in f:
                if "hash mismatch" in line and "Foo" in line:
                    found = True
        self.assertTrue(found)



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects import pdl


def run(f):
  with Context(), Location.unknown():
    module = Module.create()
    with InsertionPoint(module.body):
      print("\nTEST:", f.__name__)
      f()
    print(module)
  return f


@run
def testSequenceOp():
  sequence = transform.SequenceOp([pdl.OperationType.get()])
  with InsertionPoint(sequence.body):
    transform.YieldOp([sequence.bodyTarget])
  # CHECK-LABEL: TEST: testSequenceOp
  # CHECK: = transform.sequence {
  # CHECK: ^{{.*}}(%[[ARG0:.+]]: !pdl.operation):
  # CHECK:   yield %[[ARG0]] : !pdl.operation
  # CHECK: } : !pdl.operation


@run
def testNestedSequenceOp():
  sequence = transform.SequenceOp()
  with InsertionPoint(sequence.body):
    nested = transform.SequenceOp(sequence.bodyTarget)
    with InsertionPoint(nested.body):
      doubly_nested = transform.SequenceOp([pdl.OperationType.get()],
                                           nested.bodyTarget)
      with InsertionPoint(doubly_nested.body):
        transform.YieldOp([doubly_nested.bodyTarget])
      transform.YieldOp()
    transform.YieldOp()
  # CHECK-LABEL: TEST: testNestedSequenceOp
  # CHECK: transform.sequence {
  # CHECK: ^{{.*}}(%[[ARG0:.+]]: !pdl.operation):
  # CHECK:   sequence %[[ARG0]] {
  # CHECK:   ^{{.*}}(%[[ARG1:.+]]: !pdl.operation):
  # CHECK:     = sequence %[[ARG1]] {
  # CHECK:     ^{{.*}}(%[[ARG2:.+]]: !pdl.operation):
  # CHECK:       yield %[[ARG2]] : !pdl.operation
  # CHECK:     } : !pdl.operation
  # CHECK:   }
  # CHECK: }


@run
def testTransformPDLOps():
  withPdl = transform.WithPDLPatternsOp()
  with InsertionPoint(withPdl.body):
    sequence = transform.SequenceOp([pdl.OperationType.get()],
                                    withPdl.bodyTarget)
    with InsertionPoint(sequence.body):
      match = transform.PDLMatchOp(sequence.bodyTarget, "pdl_matcher")
      transform.YieldOp(match)
  # CHECK-LABEL: TEST: testTransformPDLOps
  # CHECK: transform.with_pdl_patterns {
  # CHECK: ^{{.*}}(%[[ARG0:.+]]: !pdl.operation):
  # CHECK:   = sequence %[[ARG0]] {
  # CHECK:   ^{{.*}}(%[[ARG1:.+]]: !pdl.operation):
  # CHECK:     %[[RES:.+]] = pdl_match @pdl_matcher in %[[ARG1]]
  # CHECK:     yield %[[RES]] : !pdl.operation
  # CHECK:   } : !pdl.operation
  # CHECK: }


@run
def testGetClosestIsolatedParentOp():
  sequence = transform.SequenceOp()
  with InsertionPoint(sequence.body):
    transform.GetClosestIsolatedParentOp(sequence.bodyTarget)
    transform.YieldOp()
  # CHECK-LABEL: TEST: testGetClosestIsolatedParentOp
  # CHECK: transform.sequence
  # CHECK: ^{{.*}}(%[[ARG1:.+]]: !pdl.operation):
  # CHECK:   = get_closest_isolated_parent %[[ARG1]]


@run
def testMergeHandlesOp():
  sequence = transform.SequenceOp()
  with InsertionPoint(sequence.body):
    transform.MergeHandlesOp([sequence.bodyTarget])
    transform.YieldOp()
  # CHECK-LABEL: TEST: testMergeHandlesOp
  # CHECK: transform.sequence
  # CHECK: ^{{.*}}(%[[ARG1:.+]]: !pdl.operation):
  # CHECK:   = merge_handles %[[ARG1]]


@run
def testReplicateOp():
  with_pdl = transform.WithPDLPatternsOp()
  with InsertionPoint(with_pdl.body):
    sequence = transform.SequenceOp(with_pdl.bodyTarget)
    with InsertionPoint(sequence.body):
      m1 = transform.PDLMatchOp(sequence.bodyTarget, "first")
      m2 = transform.PDLMatchOp(sequence.bodyTarget, "second")
      transform.ReplicateOp(m1, [m2])
      transform.YieldOp()
  # CHECK-LABEL: TEST: testReplicateOp
  # CHECK: %[[FIRST:.+]] = pdl_match
  # CHECK: %[[SECOND:.+]] = pdl_match
  # CHECK: %{{.*}} = replicate num(%[[FIRST]]) %[[SECOND]]



"""
Test python scripted process in lldb
"""

import os, shutil

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test import lldbtest

class ScriptedProcesTestCase(TestBase):

    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)

    def tearDown(self):
        TestBase.tearDown(self)

    def test_python_plugin_package(self):
        """Test that the lldb python module has a `plugins.scripted_process`
        package."""
        self.expect('script import lldb.plugins',
                    substrs=["ModuleNotFoundError"], matching=False)

        self.expect('script dir(lldb.plugins)',
                    substrs=["scripted_process"])

        self.expect('script import lldb.plugins.scripted_process',
                    substrs=["ModuleNotFoundError"], matching=False)

        self.expect('script dir(lldb.plugins.scripted_process)',
                    substrs=["ScriptedProcess"])

        self.expect('script from lldb.plugins.scripted_process import ScriptedProcess',
                    substrs=["ImportError"], matching=False)

        self.expect('script dir(ScriptedProcess)',
                    substrs=["launch"])

    def move_blueprint_to_dsym(self, blueprint_name):
        blueprint_origin_path = os.path.join(self.getSourceDir(), blueprint_name)
        dsym_bundle = self.getBuildArtifact("a.out.dSYM")
        blueprint_destination_path = os.path.join(dsym_bundle, "Contents",
                                                  "Resources", "Python")
        if not os.path.exists(blueprint_destination_path):
            os.mkdir(blueprint_destination_path)

        blueprint_destination_path = os.path.join(blueprint_destination_path, "a_out.py")
        shutil.copy(blueprint_origin_path, blueprint_destination_path)

    @skipUnlessDarwin
    def test_invalid_scripted_register_context(self):
        """Test that we can launch an lldb scripted process with an invalid
        Scripted Thread, with invalid register context."""
        self.build()

        os.environ['SKIP_SCRIPTED_PROCESS_LAUNCH'] = '1'
        def cleanup():
          del os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"]
        self.addTearDownHook(cleanup)

        self.runCmd("settings set target.load-script-from-symbol-file true")
        self.move_blueprint_to_dsym('invalid_scripted_process.py')
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)
        log_file = self.getBuildArtifact('thread.log')
        self.runCmd("log enable lldb thread -f " + log_file)
        self.assertTrue(os.path.isfile(log_file))

        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetProcessPluginName("ScriptedProcess")
        launch_info.SetScriptedProcessClassName("a_out.InvalidScriptedProcess")
        error = lldb.SBError()

        process = target.Launch(launch_info, error)

        self.assertSuccess(error)
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetProcessID(), 666)
        self.assertEqual(process.GetNumThreads(), 0)

        with open(log_file, 'r') as f:
            log = f.read()

        self.assertIn("Failed to get scripted thread registers data.", log)

    @skipUnlessDarwin
    def test_scripted_process_and_scripted_thread(self):
        """Test that we can launch an lldb scripted process using the SBAPI,
        check its process ID, read string from memory, check scripted thread
        id, name stop reason and register context.
        """
        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)

        os.environ['SKIP_SCRIPTED_PROCESS_LAUNCH'] = '1'
        def cleanup():
          del os.environ["SKIP_SCRIPTED_PROCESS_LAUNCH"]
        self.addTearDownHook(cleanup)

        scripted_process_example_relpath = 'dummy_scripted_process.py'
        self.runCmd("command script import " + os.path.join(self.getSourceDir(),
                                                            scripted_process_example_relpath))

        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetProcessPluginName("ScriptedProcess")
        launch_info.SetScriptedProcessClassName("dummy_scripted_process.DummyScriptedProcess")

        error = lldb.SBError()
        process = target.Launch(launch_info, error)
        self.assertTrue(process and process.IsValid(), PROCESS_IS_VALID)
        self.assertEqual(process.GetProcessID(), 42)

        self.assertEqual(process.GetNumThreads(), 1)

        thread = process.GetSelectedThread()
        self.assertTrue(thread, "Invalid thread.")
        self.assertEqual(thread.GetThreadID(), 0x19)
        self.assertEqual(thread.GetName(), "DummyScriptedThread.thread-1")
        self.assertEqual(thread.GetStopReason(), lldb.eStopReasonSignal)

        self.assertGreater(thread.GetNumFrames(), 0)

        frame = thread.GetFrameAtIndex(0)
        GPRs = None
        register_set = frame.registers # Returns an SBValueList.
        for regs in register_set:
            if 'general purpose' in regs.name.lower():
                GPRs = regs
                break

        self.assertTrue(GPRs, "Invalid General Purpose Registers Set")
        self.assertGreater(GPRs.GetNumChildren(), 0)
        for idx, reg in enumerate(GPRs, start=1):
            if idx > 21:
                break
            self.assertEqual(idx, int(reg.value, 16))

        self.assertTrue(frame.IsArtificial(), "Frame is not artificial")
        pc = frame.GetPCAddress().GetLoadAddress(target)
        self.assertEqual(pc, 0x0100001b00)



"""
Test lldb-vscode variables/stackTrace request for optimized code
"""

from __future__ import print_function

import unittest2
import vscode
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbvscode_testcase


class TestVSCode_optimized(lldbvscode_testcase.VSCodeTestCaseBase):

    @skipIfWindows
    @skipIfRemote
    def test_stack_frame_name(self):
        ''' Test optimized frame has special name suffix.
        '''
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        source = 'main.cpp'
        breakpoint_line = line_number(source, '// breakpoint 1')
        lines = [breakpoint_line]
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(len(breakpoint_ids), len(lines),
                        "expect correct number of breakpoints")
        self.continue_to_breakpoints(breakpoint_ids)
        leaf_frame = self.vscode.get_stackFrame(frameIndex=0)
        self.assertTrue(leaf_frame['name'].endswith(' [opt]'))
        parent_frame = self.vscode.get_stackFrame(frameIndex=1)
        self.assertTrue(parent_frame['name'].endswith(' [opt]'))

    @skipIfWindows
    @skipIfRemote
    def test_optimized_variable(self):
        ''' Test optimized variable value contains error.
        '''
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        source = 'main.cpp'
        breakpoint_line = line_number(source, '// breakpoint 2')
        lines = [breakpoint_line]
        # Set breakpoint in the thread function so we can step the threads
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(len(breakpoint_ids), len(lines),
                        "expect correct number of breakpoints")
        self.continue_to_breakpoints(breakpoint_ids)
        optimized_variable = self.vscode.get_local_variable('optimized')

        self.assertTrue(optimized_variable['value'].startswith('<error:'))



"""
Test that variable expressions of floating point types are evaluated correctly.
"""



import AbstractBase

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DoubleTypesExprTestCase(AbstractBase.GenericTester):

    # rdar://problem/8493023
    # test/types failures for Test*TypesExpr.py: element offset computed wrong
    # and sign error?

    def test_double_type(self):
        """Test that double-type variable expressions are evaluated correctly."""
        self.build_and_run_expr('double.cpp', set(['double']))

    @skipUnlessDarwin
    def test_double_type_from_block(self):
        """Test that double-type variables are displayed correctly from a block."""
        self.build_and_run_expr('double.cpp', set(['double']), bc=True)



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import arith
from mlir.dialects import func
from mlir.dialects import scf
from mlir.dialects import builtin


def constructAndPrintInModule(f):
  print("\nTEST:", f.__name__)
  with Context(), Location.unknown():
    module = Module.create()
    with InsertionPoint(module.body):
      f()
    print(module)
  return f


# CHECK-LABEL: TEST: testSimpleLoop
@constructAndPrintInModule
def testSimpleLoop():
  index_type = IndexType.get()

  @func.FuncOp.from_py_func(index_type, index_type, index_type)
  def simple_loop(lb, ub, step):
    loop = scf.ForOp(lb, ub, step, [lb, lb])
    with InsertionPoint(loop.body):
      scf.YieldOp(loop.inner_iter_args)
    return


# CHECK: func @simple_loop(%[[ARG0:.*]]: index, %[[ARG1:.*]]: index, %[[ARG2:.*]]: index)
# CHECK: scf.for %{{.*}} = %[[ARG0]] to %[[ARG1]] step %[[ARG2]]
# CHECK: iter_args(%[[I1:.*]] = %[[ARG0]], %[[I2:.*]] = %[[ARG0]])
# CHECK: scf.yield %[[I1]], %[[I2]]


# CHECK-LABEL: TEST: testInductionVar
@constructAndPrintInModule
def testInductionVar():
  index_type = IndexType.get()

  @func.FuncOp.from_py_func(index_type, index_type, index_type)
  def induction_var(lb, ub, step):
    loop = scf.ForOp(lb, ub, step, [lb])
    with InsertionPoint(loop.body):
      scf.YieldOp([loop.induction_variable])
    return


# CHECK: func @induction_var(%[[ARG0:.*]]: index, %[[ARG1:.*]]: index, %[[ARG2:.*]]: index)
# CHECK: scf.for %[[IV:.*]] = %[[ARG0]] to %[[ARG1]] step %[[ARG2]]
# CHECK: scf.yield %[[IV]]


@constructAndPrintInModule
def testOpsAsArguments():
  index_type = IndexType.get()
  callee = func.FuncOp(
      "callee", ([], [index_type, index_type]), visibility="private")
  f = func.FuncOp("ops_as_arguments", ([], []))
  with InsertionPoint(f.add_entry_block()):
    lb = arith.ConstantOp.create_index(0)
    ub = arith.ConstantOp.create_index(42)
    step = arith.ConstantOp.create_index(2)
    iter_args = func.CallOp(callee, [])
    loop = scf.ForOp(lb, ub, step, iter_args)
    with InsertionPoint(loop.body):
      scf.YieldOp(loop.inner_iter_args)
    func.ReturnOp([])


# CHECK-LABEL: TEST: testOpsAsArguments
# CHECK: func private @callee() -> (index, index)
# CHECK: func @ops_as_arguments() {
# CHECK:   %[[LB:.*]] = arith.constant 0
# CHECK:   %[[UB:.*]] = arith.constant 42
# CHECK:   %[[STEP:.*]] = arith.constant 2
# CHECK:   %[[ARGS:.*]]:2 = call @callee()
# CHECK:   scf.for %arg0 = %c0 to %c42 step %c2
# CHECK:   iter_args(%{{.*}} = %[[ARGS]]#0, %{{.*}} = %[[ARGS]]#1)
# CHECK:     scf.yield %{{.*}}, %{{.*}}
# CHECK:   return


@constructAndPrintInModule
def testIfWithoutElse():
  bool = IntegerType.get_signless(1)
  i32 = IntegerType.get_signless(32)

  @func.FuncOp.from_py_func(bool)
  def simple_if(cond):
    if_op = scf.IfOp(cond)
    with InsertionPoint(if_op.then_block):
      one = arith.ConstantOp(i32, 1)
      add = arith.AddIOp(one, one)
      scf.YieldOp([])
    return


# CHECK: func @simple_if(%[[ARG0:.*]]: i1)
# CHECK: scf.if %[[ARG0:.*]]
# CHECK:   %[[ONE:.*]] = arith.constant 1
# CHECK:   %[[ADD:.*]] = arith.addi %[[ONE]], %[[ONE]]
# CHECK: return


@constructAndPrintInModule
def testIfWithElse():
  bool = IntegerType.get_signless(1)
  i32 = IntegerType.get_signless(32)

  @func.FuncOp.from_py_func(bool)
  def simple_if_else(cond):
    if_op = scf.IfOp(cond, [i32, i32], hasElse=True)
    with InsertionPoint(if_op.then_block):
      x_true = arith.ConstantOp(i32, 0)
      y_true = arith.ConstantOp(i32, 1)
      scf.YieldOp([x_true, y_true])
    with InsertionPoint(if_op.else_block):
      x_false = arith.ConstantOp(i32, 2)
      y_false = arith.ConstantOp(i32, 3)
      scf.YieldOp([x_false, y_false])
    add = arith.AddIOp(if_op.results[0], if_op.results[1])
    return


# CHECK: func @simple_if_else(%[[ARG0:.*]]: i1)
# CHECK: %[[RET:.*]]:2 = scf.if %[[ARG0:.*]]
# CHECK:   %[[ZERO:.*]] = arith.constant 0
# CHECK:   %[[ONE:.*]] = arith.constant 1
# CHECK:   scf.yield %[[ZERO]], %[[ONE]]
# CHECK: } else {
# CHECK:   %[[TWO:.*]] = arith.constant 2
# CHECK:   %[[THREE:.*]] = arith.constant 3
# CHECK:   scf.yield %[[TWO]], %[[THREE]]
# CHECK: arith.addi %[[RET]]#0, %[[RET]]#1
# CHECK: return



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

#  This file contains functions to convert between Memrefs and NumPy arrays and vice-versa.

import numpy as np
import ctypes


class C128(ctypes.Structure):
  """A ctype representation for MLIR's Double Complex."""
  _fields_ = [("real", ctypes.c_double), ("imag", ctypes.c_double)]


class C64(ctypes.Structure):
  """A ctype representation for MLIR's Float Complex."""
  _fields_ = [("real", ctypes.c_float), ("imag", ctypes.c_float)]


class F16(ctypes.Structure):
  """A ctype representation for MLIR's Float16."""
  _fields_ = [("f16", ctypes.c_int16)]


def as_ctype(dtp):
  """Converts dtype to ctype."""
  if dtp is np.dtype(np.complex128):
    return C128
  if dtp is np.dtype(np.complex64):
    return C64
  if dtp is np.dtype(np.float16):
    return F16
  return np.ctypeslib.as_ctypes_type(dtp)


def to_numpy(array):
  """Converts ctypes array back to numpy dtype array."""
  if array.dtype == C128:
    return array.view("complex128")
  if array.dtype == C64:
    return array.view("complex64")
  if array.dtype == F16:
    return array.view("float16")
  return array


def make_nd_memref_descriptor(rank, dtype):

  class MemRefDescriptor(ctypes.Structure):
    """Builds an empty descriptor for the given rank/dtype, where rank>0."""

    _fields_ = [
        ("allocated", ctypes.c_longlong),
        ("aligned", ctypes.POINTER(dtype)),
        ("offset", ctypes.c_longlong),
        ("shape", ctypes.c_longlong * rank),
        ("strides", ctypes.c_longlong * rank),
    ]

  return MemRefDescriptor


def make_zero_d_memref_descriptor(dtype):

  class MemRefDescriptor(ctypes.Structure):
    """Builds an empty descriptor for the given dtype, where rank=0."""

    _fields_ = [
        ("allocated", ctypes.c_longlong),
        ("aligned", ctypes.POINTER(dtype)),
        ("offset", ctypes.c_longlong),
    ]

  return MemRefDescriptor


class UnrankedMemRefDescriptor(ctypes.Structure):
  """Creates a ctype struct for memref descriptor"""
  _fields_ = [("rank", ctypes.c_longlong), ("descriptor", ctypes.c_void_p)]


def get_ranked_memref_descriptor(nparray):
  """Returns a ranked memref descriptor for the given numpy array."""
  ctp = as_ctype(nparray.dtype)
  if nparray.ndim == 0:
    x = make_zero_d_memref_descriptor(ctp)()
    x.allocated = nparray.ctypes.data
    x.aligned = nparray.ctypes.data_as(ctypes.POINTER(ctp))
    x.offset = ctypes.c_longlong(0)
    return x

  x = make_nd_memref_descriptor(nparray.ndim, ctp)()
  x.allocated = nparray.ctypes.data
  x.aligned = nparray.ctypes.data_as(ctypes.POINTER(ctp))
  x.offset = ctypes.c_longlong(0)
  x.shape = nparray.ctypes.shape

  # Numpy uses byte quantities to express strides, MLIR OTOH uses the
  # torch abstraction which specifies strides in terms of elements.
  strides_ctype_t = ctypes.c_longlong * nparray.ndim
  x.strides = strides_ctype_t(*[x // nparray.itemsize for x in nparray.strides])
  return x


def get_unranked_memref_descriptor(nparray):
  """Returns a generic/unranked memref descriptor for the given numpy array."""
  d = UnrankedMemRefDescriptor()
  d.rank = nparray.ndim
  x = get_ranked_memref_descriptor(nparray)
  d.descriptor = ctypes.cast(ctypes.pointer(x), ctypes.c_void_p)
  return d


def unranked_memref_to_numpy(unranked_memref, np_dtype):
  """Converts unranked memrefs to numpy arrays."""
  ctp = as_ctype(np_dtype)
  descriptor = make_nd_memref_descriptor(unranked_memref[0].rank, ctp)
  val = ctypes.cast(unranked_memref[0].descriptor, ctypes.POINTER(descriptor))
  np_arr = np.ctypeslib.as_array(val[0].aligned, shape=val[0].shape)
  strided_arr = np.lib.stride_tricks.as_strided(
      np_arr,
      np.ctypeslib.as_array(val[0].shape),
      np.ctypeslib.as_array(val[0].strides) * np_arr.itemsize,
  )
  return to_numpy(strided_arr)


def ranked_memref_to_numpy(ranked_memref):
  """Converts ranked memrefs to numpy arrays."""
  np_arr = np.ctypeslib.as_array(
      ranked_memref[0].aligned, shape=ranked_memref[0].shape)
  strided_arr = np.lib.stride_tricks.as_strided(
      np_arr,
      np.ctypeslib.as_array(ranked_memref[0].shape),
      np.ctypeslib.as_array(ranked_memref[0].strides) * np_arr.itemsize,
  )
  return to_numpy(strided_arr)



import gdbremote_testcase


class GdbRemoteForkTestBase(gdbremote_testcase.GdbRemoteTestCaseBase):
    fork_regex = ("[$]T[0-9a-fA-F]{{2}}thread:p([0-9a-f]+)[.]([0-9a-f]+);.*"
                  "{}:p([0-9a-f]+)[.]([0-9a-f]+).*")
    fork_regex_nonstop = ("%Stop:T[0-9a-fA-F]{{2}}"
                          "thread:p([0-9a-f]+)[.]([0-9a-f]+);.*"
                          "{}:p([0-9a-f]+)[.]([0-9a-f]+).*")
    fork_capture = {1: "parent_pid", 2: "parent_tid",
                    3: "child_pid", 4: "child_tid"}
    stop_regex_base = "T[0-9a-fA-F]{{2}}thread:p{}.{};.*reason:signal.*"
    stop_regex = "^[$]" + stop_regex_base

    def start_fork_test(self, args, variant="fork", nonstop=False):
        self.build()
        self.prep_debug_monitor_and_inferior(inferior_args=args)
        self.add_qSupported_packets(["multiprocess+",
                                     "{}-events+".format(variant)])
        ret = self.expect_gdbremote_sequence()
        self.assertIn("{}-events+".format(variant), ret["qSupported_response"])
        self.reset_test_sequence()

        # continue and expect fork
        if nonstop:
            self.test_sequence.add_log_lines([
                "read packet: $QNonStop:1#00",
                "send packet: $OK#00",
                "read packet: $c#00",
                "send packet: $OK#00",
                {"direction": "send",
                 "regex": self.fork_regex_nonstop.format(variant),
                 "capture": self.fork_capture},
                "read packet: $vStopped#00",
                "send packet: $OK#00",
            ], True)
        else:
            self.test_sequence.add_log_lines([
                "read packet: $c#00",
                {"direction": "send", "regex": self.fork_regex.format(variant),
                 "capture": self.fork_capture},
            ], True)
        ret = self.expect_gdbremote_sequence()
        self.reset_test_sequence()

        return tuple(ret[x] for x in ("parent_pid", "parent_tid",
                                      "child_pid", "child_tid"))

    def fork_and_detach_test(self, variant, nonstop=False):
        parent_pid, parent_tid, child_pid, child_tid = (
            self.start_fork_test([variant], variant, nonstop=nonstop))

        # detach the forked child
        self.test_sequence.add_log_lines([
            "read packet: $D;{}#00".format(child_pid),
            "send packet: $OK#00",
            # verify that the current process is correct
            "read packet: $qC#00",
            "send packet: $QCp{}.{}#00".format(parent_pid, parent_tid),
            # verify that the correct processes are detached/available
            "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
            "send packet: $Eff#00",
            "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
            "send packet: $OK#00",
        ], True)
        self.expect_gdbremote_sequence()
        self.reset_test_sequence()
        return parent_pid, parent_tid

    def fork_and_follow_test(self, variant, nonstop=False):
        parent_pid, parent_tid, child_pid, child_tid = (
            self.start_fork_test([variant], variant, nonstop=nonstop))

        # switch to the forked child
        self.test_sequence.add_log_lines([
            "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
            "send packet: $OK#00",
            "read packet: $Hcp{}.{}#00".format(child_pid, child_tid),
            "send packet: $OK#00",
            # detach the parent
            "read packet: $D;{}#00".format(parent_pid),
            "send packet: $OK#00",
            # verify that the correct processes are detached/available
            "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
            "send packet: $Eff#00",
            "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
            "send packet: $OK#00",
            # then resume the child
            "read packet: $c#00",
        ], True)

        if nonstop:
            self.test_sequence.add_log_lines([
                "send packet: $OK#00",
                "send packet: %Stop:W00;process:{}#00".format(child_pid),
                "read packet: $vStopped#00",
                "send packet: $OK#00",
            ], True)
        else:
            self.test_sequence.add_log_lines([
                "send packet: $W00;process:{}#00".format(child_pid),
            ], True)
        self.expect_gdbremote_sequence()

    def detach_all_test(self, nonstop=False):
        parent_pid, parent_tid, child_pid, child_tid = (
            self.start_fork_test(["fork"], nonstop=nonstop))

        self.test_sequence.add_log_lines([
            # double-check our PIDs
            "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
            "send packet: $OK#00",
            "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
            "send packet: $OK#00",
            # detach all processes
            "read packet: $D#00",
            "send packet: $OK#00",
            # verify that both PIDs are invalid now
            "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
            "send packet: $Eff#00",
            "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
            "send packet: $Eff#00",
        ], True)
        self.expect_gdbremote_sequence()

    def vkill_test(self, kill_parent=False, kill_child=False, nonstop=False):
        assert kill_parent or kill_child
        parent_pid, parent_tid, child_pid, child_tid = (
            self.start_fork_test(["fork"], nonstop=nonstop))

        if kill_parent:
            self.test_sequence.add_log_lines([
                # kill the process
                "read packet: $vKill;{}#00".format(parent_pid),
                "send packet: $OK#00",
            ], True)
        if kill_child:
            self.test_sequence.add_log_lines([
                # kill the process
                "read packet: $vKill;{}#00".format(child_pid),
                "send packet: $OK#00",
            ], True)
        self.test_sequence.add_log_lines([
            # check child PID/TID
            "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
            "send packet: ${}#00".format("Eff" if kill_child else "OK"),
            # check parent PID/TID
            "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
            "send packet: ${}#00".format("Eff" if kill_parent else "OK"),
        ], True)
        self.expect_gdbremote_sequence()

    def resume_one_test(self, run_order, use_vCont=False, nonstop=False):
        parent_pid, parent_tid, child_pid, child_tid = (
            self.start_fork_test(["fork", "stop"], nonstop=nonstop))

        parent_expect = [
            self.stop_regex_base.format(parent_pid, parent_tid),
            "W00;process:{}#.*".format(parent_pid),
        ]
        child_expect = [
            self.stop_regex_base.format(child_pid, child_tid),
            "W00;process:{}#.*".format(child_pid),
        ]

        for x in run_order:
            if x == "parent":
                pidtid = (parent_pid, parent_tid)
                expect = parent_expect.pop(0)
            elif x == "child":
                pidtid = (child_pid, child_tid)
                expect = child_expect.pop(0)
            else:
                assert False, "unexpected x={}".format(x)

            if use_vCont:
                self.test_sequence.add_log_lines([
                    # continue the selected process
                    "read packet: $vCont;c:p{}.{}#00".format(*pidtid),
                ], True)
            else:
                self.test_sequence.add_log_lines([
                    # continue the selected process
                    "read packet: $Hcp{}.{}#00".format(*pidtid),
                    "send packet: $OK#00",
                    "read packet: $c#00",
                ], True)
            if nonstop:
                self.test_sequence.add_log_lines([
                    "send packet: $OK#00",
                    {"direction": "send", "regex": "%Stop:" + expect},
                    "read packet: $vStopped#00",
                    "send packet: $OK#00",
                ], True)
            else:
                self.test_sequence.add_log_lines([
                    {"direction": "send", "regex": "[$]" + expect},
                ], True)
            # if at least one process remained, check both PIDs
            if parent_expect or child_expect:
                self.test_sequence.add_log_lines([
                    "read packet: $Hgp{}.{}#00".format(parent_pid, parent_tid),
                    "send packet: ${}#00".format("OK" if parent_expect else "Eff"),
                    "read packet: $Hgp{}.{}#00".format(child_pid, child_tid),
                    "send packet: ${}#00".format("OK" if child_expect else "Eff"),
                ], True)
        self.expect_gdbremote_sequence()



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

#  This file contains the sparse compiler class.

from mlir import execution_engine
from mlir import ir
from mlir import passmanager
from typing import Sequence

class SparseCompiler:
  """Sparse compiler class for compiling and building MLIR modules."""

  def __init__(self, options: str, opt_level: int, shared_libs: Sequence[str]):
    pipeline = f'sparse-compiler{{{options} reassociate-fp-reductions=1 enable-index-optimizations=1}}'
    self.pipeline = pipeline
    self.opt_level = opt_level
    self.shared_libs = shared_libs

  def __call__(self, module: ir.Module):
    """Convenience application method."""
    self.compile(module)

  def compile(self, module: ir.Module):
    """Compiles the module by invoking the sparse copmiler pipeline."""
    passmanager.PassManager.parse(self.pipeline).run(module)

  def jit(self, module: ir.Module) -> execution_engine.ExecutionEngine:
    """Wraps the module in a JIT execution engine."""
    return execution_engine.ExecutionEngine(
        module, opt_level=self.opt_level, shared_libs=self.shared_libs)

  def compile_and_jit(self,
                      module: ir.Module) -> execution_engine.ExecutionEngine:
    """Compiles and jits the module."""
    self.compile(module)
    return self.jit(module)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class TestCase(TestBase):

    @no_debug_info_test
    def test_file_close_invalid_arg(self):
        self.expect("platform file close y", error=True,
                    substrs=["'y' is not a valid file descriptor."])
        self.expect("platform file close -1", error=True,
                    substrs=["'-1' is not a valid file descriptor."])



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from typing import Callable, Dict, List, Sequence, Tuple, Union

from .....ir import *

from .... import func
from .... import linalg
from .... import math
from .... import arith
from .... import complex
from ...._ods_common import get_op_result_or_value as _get_op_result_or_value, get_op_results_or_values as _get_op_results_or_values

from .scalar_expr import *
from .config import *
from .comprehension import *
import numpy as np

__all__ = [
    "emit_generic_structured_op",
    "emit_named_structured_op",
    "ValueList",
]

# Type aliases.
ValueList = Union[Sequence[Value], OpResultList]


def isa(cls: Type, ty: Type):
  try:
    cls(ty)
    return True
  except ValueError:
    return False


def prepare_common_structured_op(op_config: LinalgStructuredOpConfig,
                                 *ins: Value, outs: ValueList,
                                 **attrs: Union[Sequence[int], TypeFnType]):
  all_arg_defs = op_config.ordered_operands
  in_arg_defs = [
      d for d in all_arg_defs
      if d.kind in [OperandKind.SCALAR, OperandKind.INPUT_TENSOR]
  ]
  out_arg_defs = [
      d for d in all_arg_defs if d.kind == OperandKind.OUTPUT_TENSOR
  ]
  index_attr_arg_defs = [
      d for d in all_arg_defs if d.kind == OperandKind.INDEX_ATTR
  ]
  fn_attr_arg_defs = [
      d for d in all_arg_defs if d.kind in [
          OperandKind.UNARY_FN_ATTR, OperandKind.BINARY_FN_ATTR,
          OperandKind.TYPE_FN_ATTR
      ]
  ]

  # Verify outs is a sequence or a list of results.
  if not isinstance(outs, (Sequence, OpResultList)):
    raise ValueError(f"Expected named argument outs to have type Sequence or "
                     f"OpResultLis but got {type(outs)}")

  # Arity validation.
  if len(ins) != len(in_arg_defs):
    raise ValueError(f"Expected {len(in_arg_defs)} inputs but got "
                     f"{len(ins)} for {op_config}")
  if outs and len(outs) != len(out_arg_defs):
    raise ValueError(f"Expected {len(out_arg_defs)} outputs but got "
                     f"{len(outs)} for {op_config}")

  # Compute a replacement list for all index attribute symbols.
  expressions = []  # type: Sequence[AffineExpr]
  replacements = []  # type: Sequence[AffineExpr]
  for index_attr in index_attr_arg_defs:
    index_attr_vals = index_attr.operand_def.default_indices
    if index_attr.name in attrs:
      index_attr_vals = attrs.get(index_attr.name)
    assert index_attr_vals, "Index attribute has no value"
    if not all(isinstance(value, int) for value in index_attr_vals):
      raise ValueError(f"Attribute {index_attr.name} needs to be of type "
                       f"Sequence[int] but got {type(index_attr_vals)}")
    results = index_attr.index_attr_map.results  # type: AffineExprList
    if len(index_attr_vals) != len(results):
      raise ValueError(f"Attribute {index_attr.name} has length {len(results)} "
                       f"but got {len(index_attr_vals)} values")
    for expr, value in zip(results, index_attr_vals):
      expressions.append(expr)
      replacements.append(AffineConstantExpr.get(value))

  # Replace all index attribute symbols by their value.
  # TODO: Add support for shape symbols.
  indexing_maps = []  # type: Sequence[AffineMap]
  for curr in op_config.indexing_maps:
    for expression, replacement in zip(expressions, replacements):
      curr = curr.replace(expression, replacement, curr.n_dims, curr.n_symbols)
    indexing_maps.append(curr)

  # TODO: Linalg verification does not currently allow symbols.
  # Compress them for now and verify none are left.
  indexing_maps = AffineMap.compress_unused_symbols(indexing_maps,
                                                    Context.current)
  if any(indexing_map.n_symbols != 0 for indexing_map in indexing_maps):
    raise ValueError(f"Expected indexing_maps to use no symbols after "
                     f"replacement and compression but got {indexing_maps}")

  outs, out_types = _infer_structured_outs(op_config, in_arg_defs, ins,
                                           out_arg_defs, outs)

  result_types = [t for t in out_types if isa(RankedTensorType, t)]

  # Initialize the type dictionary with the predefined types.
  type_mapping = dict()  # type: Dict[str, Type]
  type_mapping["F32"] = F32Type.get()
  type_mapping["F64"] = F64Type.get()
  type_mapping["I32"] = IntegerType.get_signless(32)
  type_mapping["I64"] = IntegerType.get_signless(64)

  # Extract type vars for input/output based types.
  block_arg_types = list()  # type: List[Type]
  for arg_def, arg_element_type in zip(in_arg_defs + out_arg_defs,
                                       _get_types_from_values(*ins, *outs)):
    _add_type_mapping(arg_def, arg_element_type, type_mapping, block_arg_types)

  # Emit the generic op.
  # TODO: Support emission of pure memref form.
  indexing_maps_attr = ArrayAttr.get(
      [AffineMapAttr.get(am) for am in indexing_maps])
  iterator_types_attr = ArrayAttr.get(
      [StringAttr.get(s) for s in op_config.iterator_types])

  # Compute the index attributes used when emitting a named structured op.
  index_attrs = {}  # type: Dict[str, DenseElementAttr]
  for index_attr in index_attr_arg_defs:
    index_attr_vals = attrs.get(index_attr.name)
    # Only forward attributes set to a non-default value.
    if index_attr_vals:
      array = np.array(index_attr_vals, dtype=np.int64)
      index_attrs[index_attr.name] = DenseElementsAttr.get(array)

  # Compute the function attribute mapping.
  fn_attr_mapping = {}
  for fn_attr in fn_attr_arg_defs:
    attr_val = fn_attr.operand_def.default_fn
    attr_kind = fn_attr.kind
    if fn_attr.name in attrs:
      fn = attrs.get(fn_attr.name)
      if attr_kind == OperandKind.UNARY_FN_ATTR:
        if not isinstance(fn, UnaryFnType):
          raise ValueError(f"Attribute {fn_attr.name} needs to be of type "
                           f"UnaryFnType but got {type(attr_val)}")
      elif attr_kind == OperandKind.BINARY_FN_ATTR:
        if not isinstance(fn, BinaryFnType):
          raise ValueError(f"Attribute {fn_attr.name} needs to be of type "
                           f"BinaryFnType but got {type(attr_val)}")
      else:
        if not isinstance(fn, TypeFnType):
          raise ValueError(f"Attribute {fn_attr.name} needs to be of type "
                           f"TypeFnType but got {type(attr_val)}")
      attr_val = fn.fn_name
    assert attr_val, "Function attribute has no value"
    fn_attr_mapping[fn_attr.name] = (attr_val, attr_kind)

  return (all_arg_defs, in_arg_defs, out_arg_defs, outs, result_types,
          type_mapping, indexing_maps_attr, iterator_types_attr, index_attrs,
          fn_attr_mapping, block_arg_types)


def emit_generic_structured_op(op_config: LinalgStructuredOpConfig, *ins: Value,
                               outs: ValueList, **attrs: Sequence[int]):
  all_arg_defs, in_arg_defs, out_arg_defs, outs, result_types, type_mapping, \
  indexing_maps_attr, iterator_types_attr, index_attrs, fn_attr_mapping, \
  block_arg_types = \
     prepare_common_structured_op(op_config, *ins, outs = outs, **attrs)

  # An operation that accesses only scalars and scalar/rank zero tensors is
  # rank polymorhpic. We implement rank polymorphism by generating different
  # indexing maps and iterators that match the rank of the first output tensor.
  # An operation is rank polymorphic if the iteration domain has rank zero.
  if not iterator_types_attr:
    rank = ShapedType(outs[0].type).rank
    iterator_types_attr = ArrayAttr.get([StringAttr.get("parallel")] * rank)
    scalar_map = AffineMap.get(rank, 0, [])
    tensor_map = AffineMap.get_identity(rank)
    indexing_maps = []
    for arg_def in all_arg_defs:
      if arg_def.operand_def.kind == OperandKind.SCALAR:
        indexing_maps.append(scalar_map)
      if arg_def.operand_def.is_tensor():
        idx = arg_def.operand_def.registered_index
        if idx < len(ins) and ShapedType(ins[idx].type).rank == 0:
          indexing_maps.append(scalar_map)
        else:
          indexing_maps.append(tensor_map)
    indexing_maps_attr = ArrayAttr.get(
        [AffineMapAttr.get(am) for am in indexing_maps])

  generic_op = linalg.GenericOp(
      result_tensors=result_types,
      inputs=ins,
      outputs=outs,
      indexing_maps=indexing_maps_attr,
      iterator_types=iterator_types_attr,
      doc=None,  # TODO: Make optional.
      library_call=None)  # TODO: Make optional.

  # Construct the body.
  block_arg_names = _get_operand_def_names(*in_arg_defs, *out_arg_defs)
  block = generic_op.regions[0].blocks.append(*block_arg_types)
  block_arg_mapping = dict(zip(block_arg_names, block.arguments))
  with InsertionPoint(block):
    body_builder = _BodyBuilder(type_mapping, block_arg_mapping,
                                fn_attr_mapping)
    for assignment in op_config.assignments:
      body_builder.assign(assignment)
    body_builder.yield_outputs(*_get_operand_def_names(*out_arg_defs))

  if len(result_types) == 1:
    return generic_op.result
  else:
    return generic_op.results


def emit_named_structured_op(op_config: LinalgStructuredOpConfig, op_name: str,
                             op_class_name: str, *ins: Value, outs: ValueList,
                             **attrs: Sequence[int]):
  all_arg_defs, in_arg_defs, out_arg_defs, outs, result_types, type_mapping, \
  indexing_maps_attr, iterator_types_attr, index_attrs, fn_attr_mapping, \
  block_arg_types = \
     prepare_common_structured_op(op_config, *ins, outs = outs, **attrs)

  # If we get here, there must exist a builtin class `op_class_name`.
  ctx = Context.current
  fully_qualified_name = "linalg." + op_name
  if (not ctx.is_registered_operation(fully_qualified_name) or
      not op_class_name in linalg.__dict__.keys()):
    raise NotImplementedError(
        f"Unknown named op_name / op_class_name: {op_name} / {op_class_name}")

  # Set the index attributes used to compute the indexing maps.
  named_op = getattr(linalg, op_class_name)(ins, outs, result_types)
  for name, value in index_attrs.items():
    named_op.operation.attributes[name] = value

  # Compute the function attributes by combining operand kind and function name.
  for name, (fn_name, kind) in fn_attr_mapping.items():
    assert kind.name.lower().endswith("_attr")
    enum_name = kind.name.lower()[:-5]
    named_op.operation.attributes[name] = Attribute.parse(
        f"#linalg.{enum_name}<{fn_name}>")

  linalg.fill_builtin_region(named_op.operation)

  if len(result_types) == 1:
    return named_op.result
  else:
    return named_op.results


class _BodyBuilder:
  """Constructs a structured op body by evaluating assignments."""

  def __init__(self, type_mapping: Dict[str, Type],
               block_arg_mapping: Dict[str, Value], fn_attr_mapping: Dict[str,
                                                                          str]):
    self.type_mapping = type_mapping
    self.block_arg_mapping = block_arg_mapping
    self.fn_attr_mapping = fn_attr_mapping
    self.yield_mapping = dict()  # type: Dict[str, Value]

  def assign(self, assignment: ScalarAssign):
    if assignment.arg in self.yield_mapping:
      raise ValueError(
          f"Multiple assignments to the same argument are forbidden: "
          f"{assignment}")
    self.yield_mapping[assignment.arg] = self.expression(assignment.value)

  def expression(self, expr: ScalarExpression) -> Value:
    if expr.scalar_arg:
      try:
        return self.block_arg_mapping[expr.scalar_arg.arg]
      except KeyError:
        raise ValueError(f"Argument {expr.scalar_arg.arg} is not bound for "
                         f"this structured op.")
    elif expr.scalar_const:
      value_attr = Attribute.parse(expr.scalar_const.value)
      return arith.ConstantOp(value_attr.type, value_attr).result
    elif expr.scalar_index:
      dim_attr = IntegerAttr.get(
          IntegerType.get_signless(64), expr.scalar_index.dim)
      return linalg.IndexOp(dim_attr).result
    elif expr.scalar_fn:
      kind = expr.scalar_fn.kind.name.lower()
      fn_name = expr.scalar_fn.fn_name
      if expr.scalar_fn.attr_name:
        fn_name, _ = self.fn_attr_mapping[expr.scalar_fn.attr_name]
      fn = self._get_function(f"_{kind}_{fn_name}")
      operand_values = [
          self.expression(operand) for operand in expr.scalar_fn.operands
      ]
      if expr.scalar_fn.kind == FunctionKind.TYPE:
        operand_values = [expr.scalar_fn.type_var.name] + operand_values
      return fn(*operand_values)
    raise NotImplementedError(f"Unimplemented scalar body expression: {expr}")

  def yield_outputs(self, *output_names: str):
    output_values = []
    for n in output_names:
      try:
        output_values.append(self.yield_mapping[n])
      except KeyError:
        raise ValueError(f"Body assignments do not assign all outputs: "
                         f"missing '{n}'")
    linalg.YieldOp(output_values)

  def _get_function(self, fn_name: str) -> Callable:
    try:
      fn = getattr(self, f"{fn_name}")
    except AttributeError:
      raise ValueError(f"Function '{fn_name}' is not a known function")
    return fn

  def _cast(self,
            type_var_name: str,
            operand: Value,
            is_unsigned_cast: bool = False) -> Value:
    try:
      to_type = self.type_mapping[type_var_name]
    except KeyError:
      raise ValueError(f"Unbound type variable '{type_var_name}' ("
                       f"expected one of {self.type_mapping.keys()}")
    if operand.type == to_type:
      return operand
    if _is_integer_type(to_type):
      return self._cast_to_integer(to_type, operand, is_unsigned_cast)
    elif _is_floating_point_type(to_type):
      return self._cast_to_floating_point(to_type, operand, is_unsigned_cast)

  def _cast_to_integer(self, to_type: Type, operand: Value,
                       is_unsigned_cast: bool) -> Value:
    to_width = IntegerType(to_type).width
    operand_type = operand.type
    if _is_floating_point_type(operand_type):
      if is_unsigned_cast:
        return arith.FPToUIOp(to_type, operand).result
      return arith.FPToSIOp(to_type, operand).result
    if _is_index_type(operand_type):
      return arith.IndexCastOp(to_type, operand).result
    # Assume integer.
    from_width = IntegerType(operand_type).width
    if to_width > from_width:
      if is_unsigned_cast:
        return arith.ExtUIOp(to_type, operand).result
      return arith.ExtSIOp(to_type, operand).result
    elif to_width < from_width:
      return arith.TruncIOp(to_type, operand).result
    raise ValueError(f"Unable to cast body expression from {operand_type} to "
                     f"{to_type}")

  def _cast_to_floating_point(self, to_type: Type, operand: Value,
                              is_unsigned_cast: bool) -> Value:
    operand_type = operand.type
    if _is_integer_type(operand_type):
      if is_unsigned_cast:
        return arith.UIToFPOp(to_type, operand).result
      return arith.SIToFPOp(to_type, operand).result
    # Assume FloatType.
    to_width = _get_floating_point_width(to_type)
    from_width = _get_floating_point_width(operand_type)
    if to_width > from_width:
      return arith.ExtFOp(to_type, operand).result
    elif to_width < from_width:
      return arith.TruncFOp(to_type, operand).result
    raise ValueError(f"Unable to cast body expression from {operand_type} to "
                     f"{to_type}")

  def _type_cast_signed(self, type_var_name: str, operand: Value) -> Value:
    return self._cast(type_var_name, operand, False)

  def _type_cast_unsigned(self, type_var_name: str, operand: Value) -> Value:
    return self._cast(type_var_name, operand, True)

  def _unary_exp(self, x: Value) -> Value:
    if _is_floating_point_type(x.type):
      return math.ExpOp(x).result
    raise NotImplementedError("Unsupported 'exp' operand: {x}")

  def _unary_log(self, x: Value) -> Value:
    if _is_floating_point_type(x.type):
      return math.LogOp(x).result
    raise NotImplementedError("Unsupported 'log' operand: {x}")

  def _unary_abs(self, x: Value) -> Value:
    if _is_floating_point_type(x.type):
      return math.AbsOp(x).result
    raise NotImplementedError("Unsupported 'abs' operand: {x}")

  def _unary_ceil(self, x: Value) -> Value:
    if _is_floating_point_type(x.type):
      return math.CeilOp(x).result
    raise NotImplementedError("Unsupported 'ceil' operand: {x}")

  def _unary_floor(self, x: Value) -> Value:
    if _is_floating_point_type(x.type):
      return math.FloorOp(x).result
    raise NotImplementedError("Unsupported 'floor' operand: {x}")

  def _unary_negf(self, x: Value) -> Value:
    if _is_floating_point_type(x.type):
      return arith.NegFOp(x).result
    if _is_complex_type(x.type):
      return complex.NegOp(x).result
    raise NotImplementedError("Unsupported 'negf' operand: {x}")

  def _binary_add(self, lhs: Value, rhs: Value) -> Value:
    if _is_floating_point_type(lhs.type):
      return arith.AddFOp(lhs, rhs).result
    if _is_integer_type(lhs.type) or _is_index_type(lhs.type):
      return arith.AddIOp(lhs, rhs).result
    if _is_complex_type(lhs.type):
      return complex.AddOp(lhs, rhs).result
    raise NotImplementedError("Unsupported 'add' operands: {lhs}, {rhs}")

  def _binary_sub(self, lhs: Value, rhs: Value) -> Value:
    if _is_floating_point_type(lhs.type):
      return arith.SubFOp(lhs, rhs).result
    if _is_integer_type(lhs.type) or _is_index_type(lhs.type):
      return arith.SubIOp(lhs, rhs).result
    if _is_complex_type(lhs.type):
      return complex.SubOp(lhs, rhs).result
    raise NotImplementedError("Unsupported 'sub' operands: {lhs}, {rhs}")

  def _binary_mul(self, lhs: Value, rhs: Value) -> Value:
    if _is_floating_point_type(lhs.type):
      return arith.MulFOp(lhs, rhs).result
    if _is_integer_type(lhs.type) or _is_index_type(lhs.type):
      return arith.MulIOp(lhs, rhs).result
    if _is_complex_type(lhs.type):
      return complex.MulOp(lhs, rhs).result
    raise NotImplementedError("Unsupported 'mul' operands: {lhs}, {rhs}")

  def _binary_max_signed(self, lhs: Value, rhs: Value) -> Value:
    if _is_floating_point_type(lhs.type):
      return arith.MaxFOp(lhs, rhs).result
    if _is_integer_type(lhs.type) or _is_index_type(lhs.type):
      return arith.MaxSIOp(lhs, rhs).result
    raise NotImplementedError("Unsupported 'max' operands: {lhs}, {rhs}")

  def _binary_max_unsigned(self, lhs: Value, rhs: Value) -> Value:
    if _is_floating_point_type(lhs.type):
      return arith.MaxFOp(lhs, rhs).result
    if _is_integer_type(lhs.type) or _is_index_type(lhs.type):
      return arith.MaxUIOp(lhs, rhs).result
    raise NotImplementedError(
        "Unsupported 'max_unsigned' operands: {lhs}, {rhs}")

  def _binary_min_signed(self, lhs: Value, rhs: Value) -> Value:
    if _is_floating_point_type(lhs.type):
      return arith.MinFOp(lhs, rhs).result
    if _is_integer_type(lhs.type) or _is_index_type(lhs.type):
      return arith.MinSIOp(lhs, rhs).result
    raise NotImplementedError("Unsupported 'min' operands: {lhs}, {rhs}")

  def _binary_min_unsigned(self, lhs: Value, rhs: Value) -> Value:
    if _is_floating_point_type(lhs.type):
      return arith.MinFOp(lhs, rhs).result
    if _is_integer_type(lhs.type) or _is_index_type(lhs.type):
      return arith.MinUIOp(lhs, rhs).result
    raise NotImplementedError(
        "Unsupported 'min_unsigned' operands: {lhs}, {rhs}")


def _infer_structured_outs(
    op_config: LinalgStructuredOpConfig,
    in_arg_defs: Sequence[OperandDefConfig], ins: Sequence[Value],
    out_arg_defs: Sequence[OperandDefConfig],
    outs: Union[Sequence[Value], OpResultList]) -> Tuple[ValueList, List[Type]]:
  """Infers implicit outs and output types.

  Respects existing contents of outs if not empty.

  Returns:
    normalized outs, output types
  """
  # If outs were explicitly provided, we accept them verbatim.
  if outs:
    return outs, [out.type for out in outs]

  raise NotImplementedError(f"Output tensor inference not yet supported for "
                            "structured ops")


def _get_types_from_values(*values: Value) -> Sequence[Type]:
  types = []
  for v in values:
    types.append(v.type)
  return types


def _get_operand_def_names(*operand_configs: OperandDefConfig) -> Sequence[str]:
  return [odc.operand_def.name for odc in operand_configs]


def _add_type_mapping(operand_config: OperandDefConfig, operand_type: Type,
                      type_mapping: Dict[str, Type],
                      block_arg_types: Sequence[Type]):
  element_or_self_type = operand_type
  # Get the element type for tensor operands and the type itself for scalars.
  if operand_config.shape_map:
    try:
      element_or_self_type = ShapedType(operand_type).element_type
    except Exception as e:
      raise ValueError(f"Expected ShapedType but got {operand_type}") from e
  name = operand_config.type_var.name
  if name in type_mapping:
    if type_mapping[name] != element_or_self_type:
      raise ValueError(f"Cannot overwrite type mapping {name} = "
                       f"{type_mapping[name]} by type {element_or_self_type}")
  type_mapping[name] = element_or_self_type
  block_arg_types.append(element_or_self_type)


def _is_complex_type(t: Type) -> bool:
  return ComplexType.isinstance(t)


def _is_floating_point_type(t: Type) -> bool:
  # TODO: Create a FloatType in the Python API and implement the switch
  # there.
  return (F64Type.isinstance(t) or F32Type.isinstance(t) or
          F16Type.isinstance(t) or BF16Type.isinstance(t))


def _is_integer_type(t: Type) -> bool:
  return IntegerType.isinstance(t)


def _is_index_type(t: Type) -> bool:
  return IndexType.isinstance(t)


def _get_floating_point_width(t: Type) -> int:
  # TODO: Create a FloatType in the Python API and implement the switch
  # there.
  if F64Type.isinstance(t):
    return 64
  if F32Type.isinstance(t):
    return 32
  if F16Type.isinstance(t):
    return 16
  if BF16Type.isinstance(t):
    return 16
  raise NotImplementedError(f"Unhandled floating point type switch {t}")



"""
Test 'watchpoint command'.
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class WatchpointLLDBCommandTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Our simple source filename.
        self.source = 'main.cpp'
        # Find the line number to break inside main().
        self.line = line_number(
            self.source, '// Set break point at this line.')
        # And the watchpoint variable declaration line number.
        self.decl = line_number(self.source,
                                '// Watchpoint variable declaration.')
        # Build dictionary to have unique executable names for each test
        # method.
        self.exe_name = 'a%d.out' % self.test_number
        self.d = {'CXX_SOURCES': self.source, 'EXE': self.exe_name}

    def test_watchpoint_command(self):
        """Test 'watchpoint command'."""
        self.build(dictionary=self.d)
        self.setTearDownCleanup(dictionary=self.d)

        exe = self.getBuildArtifact(self.exe_name)
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Add a breakpoint to set a watchpoint when stopped on the breakpoint.
        lldbutil.run_break_set_by_file_and_line(
            self, None, self.line, num_expected_locations=1)

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should be stopped again due to the breakpoint.
        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # Now let's set a write-type watchpoint for 'global'.
        self.expect(
            "watchpoint set variable -w write global",
            WATCHPOINT_CREATED,
            substrs=[
                'Watchpoint created',
                'size = 4',
                'type = w',
                '%s:%d' %
                (self.source,
                 self.decl)])

        self.runCmd('watchpoint command add 1 -o "expr -- cookie = 777"')

        # List the watchpoint command we just added.
        self.expect("watchpoint command list 1",
                    substrs=['expr -- cookie = 777'])

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect("watchpoint list -v",
                    substrs=['hit_count = 0'])

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (write type).
        # The stop reason of the thread should be watchpoint.
        self.expect("thread backtrace", STOPPED_DUE_TO_WATCHPOINT,
                    substrs=['stop reason = watchpoint'])

        # Check that the watchpoint snapshoting mechanism is working.
        self.expect(
            "watchpoint list -v",
            substrs=[
                'old value: 0', 'new value: 1', 'hit_count = 1',
                'ignore_count = 0'
            ])

        # The watchpoint command "forced" our global variable 'cookie' to
        # become 777.
        self.expect("frame variable --show-globals cookie",
                    substrs=['(int32_t)', 'cookie = 777'])

    def test_watchpoint_command_can_disable_a_watchpoint(self):
        """Test that 'watchpoint command' action can disable a watchpoint after it is triggered."""
        self.build(dictionary=self.d)
        self.setTearDownCleanup(dictionary=self.d)

        exe = self.getBuildArtifact(self.exe_name)
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Add a breakpoint to set a watchpoint when stopped on the breakpoint.
        lldbutil.run_break_set_by_file_and_line(
            self, None, self.line, num_expected_locations=1)

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should be stopped again due to the breakpoint.
        # The stop reason of the thread should be breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # Now let's set a write-type watchpoint for 'global'.
        self.expect(
            "watchpoint set variable -w write global",
            WATCHPOINT_CREATED,
            substrs=[
                'Watchpoint created',
                'size = 4',
                'type = w',
                '%s:%d' %
                (self.source,
                 self.decl)])

        self.runCmd('watchpoint command add 1 -o "watchpoint disable 1"')

        # List the watchpoint command we just added.
        self.expect("watchpoint command list 1",
                    substrs=['watchpoint disable 1'])

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect("watchpoint list -v",
                    substrs=['hit_count = 0'])

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (write type).
        # The stop reason of the thread should be watchpoint.
        self.expect("thread backtrace", STOPPED_DUE_TO_WATCHPOINT,
                    substrs=['stop reason = watchpoint'])

        # Check that the watchpoint has been disabled.
        self.expect("watchpoint list -v",
                    substrs=['disabled'])

        self.runCmd("process continue")

        # There should be no more watchpoint hit and the process status should
        # be 'exited'.
        self.expect("process status",
                    substrs=['exited'])



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):

    @skipUnlessDarwin
    @no_debug_info_test
    def test_keyword(self):
      # Make sure that C++ keywords work in the expression parser when using
      # Objective-C++.
      self.expect("expr -l objective-c++ -- constexpr int i = 3 + 3; i", substrs=["= 6"])



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
from dex.dextIR.LocIR import LocIR


class FrameIR:
    """Data class which represents a frame in the call stack"""

    def __init__(self, function: str, is_inlined: bool, loc: LocIR):
        self.function = function
        self.is_inlined = is_inlined
        self.loc = loc



# RUN: %PYTHON %s 2>&1 | FileCheck %s
# REQUIRES: host-supports-jit
import gc, sys
from mlir.ir import *
from mlir.passmanager import *
from mlir.execution_engine import *
from mlir.runtime import *


# Log everything to stderr and flush so that we have a unified stream to match
# errors/info emitted by MLIR to stderr.
def log(*args):
  print(*args, file=sys.stderr)
  sys.stderr.flush()


def run(f):
  log("\nTEST:", f.__name__)
  f()
  gc.collect()
  assert Context._get_live_count() == 0


# Verify capsule interop.
# CHECK-LABEL: TEST: testCapsule
def testCapsule():
  with Context():
    module = Module.parse(r"""
llvm.func @none() {
  llvm.return
}
    """)
    execution_engine = ExecutionEngine(module)
    execution_engine_capsule = execution_engine._CAPIPtr
    # CHECK: mlir.execution_engine.ExecutionEngine._CAPIPtr
    log(repr(execution_engine_capsule))
    execution_engine._testing_release()
    execution_engine1 = ExecutionEngine._CAPICreate(execution_engine_capsule)
    # CHECK: _mlirExecutionEngine.ExecutionEngine
    log(repr(execution_engine1))


run(testCapsule)


# Test invalid ExecutionEngine creation
# CHECK-LABEL: TEST: testInvalidModule
def testInvalidModule():
  with Context():
    # Builtin function
    module = Module.parse(r"""
    func.func @foo() { return }
    """)
    # CHECK: Got RuntimeError:  Failure while creating the ExecutionEngine.
    try:
      execution_engine = ExecutionEngine(module)
    except RuntimeError as e:
      log("Got RuntimeError: ", e)


run(testInvalidModule)


def lowerToLLVM(module):
  pm = PassManager.parse(
      "convert-complex-to-llvm,convert-memref-to-llvm,convert-func-to-llvm,reconcile-unrealized-casts")
  pm.run(module)
  return module


# Test simple ExecutionEngine execution
# CHECK-LABEL: TEST: testInvokeVoid
def testInvokeVoid():
  with Context():
    module = Module.parse(r"""
func.func @void() attributes { llvm.emit_c_interface } {
  return
}
    """)
    execution_engine = ExecutionEngine(lowerToLLVM(module))
    # Nothing to check other than no exception thrown here.
    execution_engine.invoke("void")


run(testInvokeVoid)


# Test argument passing and result with a simple float addition.
# CHECK-LABEL: TEST: testInvokeFloatAdd
def testInvokeFloatAdd():
  with Context():
    module = Module.parse(r"""
func.func @add(%arg0: f32, %arg1: f32) -> f32 attributes { llvm.emit_c_interface } {
  %add = arith.addf %arg0, %arg1 : f32
  return %add : f32
}
    """)
    execution_engine = ExecutionEngine(lowerToLLVM(module))
    # Prepare arguments: two input floats and one result.
    # Arguments must be passed as pointers.
    c_float_p = ctypes.c_float * 1
    arg0 = c_float_p(42.)
    arg1 = c_float_p(2.)
    res = c_float_p(-1.)
    execution_engine.invoke("add", arg0, arg1, res)
    # CHECK: 42.0 + 2.0 = 44.0
    log("{0} + {1} = {2}".format(arg0[0], arg1[0], res[0]))


run(testInvokeFloatAdd)


# Test callback
# CHECK-LABEL: TEST: testBasicCallback
def testBasicCallback():
  # Define a callback function that takes a float and an integer and returns a float.
  @ctypes.CFUNCTYPE(ctypes.c_float, ctypes.c_float, ctypes.c_int)
  def callback(a, b):
    return a / 2 + b / 2

  with Context():
    # The module just forwards to a runtime function known as "some_callback_into_python".
    module = Module.parse(r"""
func.func @add(%arg0: f32, %arg1: i32) -> f32 attributes { llvm.emit_c_interface } {
  %resf = call @some_callback_into_python(%arg0, %arg1) : (f32, i32) -> (f32)
  return %resf : f32
}
func.func private @some_callback_into_python(f32, i32) -> f32 attributes { llvm.emit_c_interface }
    """)
    execution_engine = ExecutionEngine(lowerToLLVM(module))
    execution_engine.register_runtime("some_callback_into_python", callback)

    # Prepare arguments: two input floats and one result.
    # Arguments must be passed as pointers.
    c_float_p = ctypes.c_float * 1
    c_int_p = ctypes.c_int * 1
    arg0 = c_float_p(42.)
    arg1 = c_int_p(2)
    res = c_float_p(-1.)
    execution_engine.invoke("add", arg0, arg1, res)
    # CHECK: 42.0 + 2 = 44.0
    log("{0} + {1} = {2}".format(arg0[0], arg1[0], res[0] * 2))


run(testBasicCallback)


# Test callback with an unranked memref
# CHECK-LABEL: TEST: testUnrankedMemRefCallback
def testUnrankedMemRefCallback():
  # Define a callback function that takes an unranked memref, converts it to a numpy array and prints it.
  @ctypes.CFUNCTYPE(None, ctypes.POINTER(UnrankedMemRefDescriptor))
  def callback(a):
    arr = unranked_memref_to_numpy(a, np.float32)
    log("Inside callback: ")
    log(arr)

  with Context():
    # The module just forwards to a runtime function known as "some_callback_into_python".
    module = Module.parse(r"""
func.func @callback_memref(%arg0: memref<*xf32>) attributes { llvm.emit_c_interface } {
  call @some_callback_into_python(%arg0) : (memref<*xf32>) -> ()
  return
}
func.func private @some_callback_into_python(memref<*xf32>) -> () attributes { llvm.emit_c_interface }
""")
    execution_engine = ExecutionEngine(lowerToLLVM(module))
    execution_engine.register_runtime("some_callback_into_python", callback)
    inp_arr = np.array([[1.0, 2.0], [3.0, 4.0]], np.float32)
    # CHECK: Inside callback:
    # CHECK{LITERAL}: [[1. 2.]
    # CHECK{LITERAL}:  [3. 4.]]
    execution_engine.invoke(
        "callback_memref",
        ctypes.pointer(ctypes.pointer(get_unranked_memref_descriptor(inp_arr))),
    )
    inp_arr_1 = np.array([5, 6, 7], dtype=np.float32)
    strided_arr = np.lib.stride_tricks.as_strided(
        inp_arr_1, strides=(4, 0), shape=(3, 4))
    # CHECK: Inside callback:
    # CHECK{LITERAL}: [[5. 5. 5. 5.]
    # CHECK{LITERAL}:  [6. 6. 6. 6.]
    # CHECK{LITERAL}:  [7. 7. 7. 7.]]
    execution_engine.invoke(
        "callback_memref",
        ctypes.pointer(
            ctypes.pointer(get_unranked_memref_descriptor(strided_arr))),
    )


run(testUnrankedMemRefCallback)


# Test callback with a ranked memref.
# CHECK-LABEL: TEST: testRankedMemRefCallback
def testRankedMemRefCallback():
  # Define a callback function that takes a ranked memref, converts it to a numpy array and prints it.
  @ctypes.CFUNCTYPE(
      None,
      ctypes.POINTER(
          make_nd_memref_descriptor(2,
                                    np.ctypeslib.as_ctypes_type(np.float32))),
  )
  def callback(a):
    arr = ranked_memref_to_numpy(a)
    log("Inside Callback: ")
    log(arr)

  with Context():
    # The module just forwards to a runtime function known as "some_callback_into_python".
    module = Module.parse(r"""
func.func @callback_memref(%arg0: memref<2x2xf32>) attributes { llvm.emit_c_interface } {
  call @some_callback_into_python(%arg0) : (memref<2x2xf32>) -> ()
  return
}
func.func private @some_callback_into_python(memref<2x2xf32>) -> () attributes { llvm.emit_c_interface }
""")
    execution_engine = ExecutionEngine(lowerToLLVM(module))
    execution_engine.register_runtime("some_callback_into_python", callback)
    inp_arr = np.array([[1.0, 5.0], [6.0, 7.0]], np.float32)
    # CHECK: Inside Callback:
    # CHECK{LITERAL}: [[1. 5.]
    # CHECK{LITERAL}:  [6. 7.]]
    execution_engine.invoke(
        "callback_memref",
        ctypes.pointer(ctypes.pointer(get_ranked_memref_descriptor(inp_arr))))


run(testRankedMemRefCallback)


#  Test addition of two memrefs.
# CHECK-LABEL: TEST: testMemrefAdd
def testMemrefAdd():
  with Context():
    module = Module.parse("""
    module  {
      func.func @main(%arg0: memref<1xf32>, %arg1: memref<f32>, %arg2: memref<1xf32>) attributes { llvm.emit_c_interface } {
        %0 = arith.constant 0 : index
        %1 = memref.load %arg0[%0] : memref<1xf32>
        %2 = memref.load %arg1[] : memref<f32>
        %3 = arith.addf %1, %2 : f32
        memref.store %3, %arg2[%0] : memref<1xf32>
        return
      }
    } """)
    arg1 = np.array([32.5]).astype(np.float32)
    arg2 = np.array(6).astype(np.float32)
    res = np.array([0]).astype(np.float32)

    arg1_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg1)))
    arg2_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg2)))
    res_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(res)))

    execution_engine = ExecutionEngine(lowerToLLVM(module))
    execution_engine.invoke("main", arg1_memref_ptr, arg2_memref_ptr,
                            res_memref_ptr)
    # CHECK: [32.5] + 6.0 = [38.5]
    log("{0} + {1} = {2}".format(arg1, arg2, res))


run(testMemrefAdd)


# Test addition of two f16 memrefs
# CHECK-LABEL: TEST: testF16MemrefAdd
def testF16MemrefAdd():
  with Context():
    module = Module.parse("""
    module  {
      func.func @main(%arg0: memref<1xf16>,
                      %arg1: memref<1xf16>,
                      %arg2: memref<1xf16>) attributes { llvm.emit_c_interface } {
        %0 = arith.constant 0 : index
        %1 = memref.load %arg0[%0] : memref<1xf16>
        %2 = memref.load %arg1[%0] : memref<1xf16>
        %3 = arith.addf %1, %2 : f16
        memref.store %3, %arg2[%0] : memref<1xf16>
        return
      }
    } """)

    arg1 = np.array([11.]).astype(np.float16)
    arg2 = np.array([22.]).astype(np.float16)
    arg3 = np.array([0.]).astype(np.float16)

    arg1_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg1)))
    arg2_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg2)))
    arg3_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg3)))

    execution_engine = ExecutionEngine(lowerToLLVM(module))
    execution_engine.invoke("main", arg1_memref_ptr, arg2_memref_ptr,
                            arg3_memref_ptr)
    # CHECK: [11.] + [22.] = [33.]
    log("{0} + {1} = {2}".format(arg1, arg2, arg3))

    # test to-numpy utility
    # CHECK: [33.]
    npout = ranked_memref_to_numpy(arg3_memref_ptr[0])
    log(npout)


run(testF16MemrefAdd)


# Test addition of two complex memrefs
# CHECK-LABEL: TEST: testComplexMemrefAdd
def testComplexMemrefAdd():
  with Context():
    module = Module.parse("""
    module  {
      func.func @main(%arg0: memref<1xcomplex<f64>>,
                      %arg1: memref<1xcomplex<f64>>,
                      %arg2: memref<1xcomplex<f64>>) attributes { llvm.emit_c_interface } {
        %0 = arith.constant 0 : index
        %1 = memref.load %arg0[%0] : memref<1xcomplex<f64>>
        %2 = memref.load %arg1[%0] : memref<1xcomplex<f64>>
        %3 = complex.add %1, %2 : complex<f64>
        memref.store %3, %arg2[%0] : memref<1xcomplex<f64>>
        return
      }
    } """)

    arg1 = np.array([1.+2.j]).astype(np.complex128)
    arg2 = np.array([3.+4.j]).astype(np.complex128)
    arg3  = np.array([0.+0.j]).astype(np.complex128)

    arg1_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg1)))
    arg2_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg2)))
    arg3_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg3)))

    execution_engine = ExecutionEngine(lowerToLLVM(module))
    execution_engine.invoke("main",
                            arg1_memref_ptr,
                            arg2_memref_ptr,
                            arg3_memref_ptr)
    # CHECK: [1.+2.j] + [3.+4.j] = [4.+6.j]
    log("{0} + {1} = {2}".format(arg1, arg2, arg3))

    # test to-numpy utility
    # CHECK: [4.+6.j]
    npout = ranked_memref_to_numpy(arg3_memref_ptr[0])
    log(npout)


run(testComplexMemrefAdd)


# Test addition of two complex unranked memrefs
# CHECK-LABEL: TEST: testComplexUnrankedMemrefAdd
def testComplexUnrankedMemrefAdd():
  with Context():
    module = Module.parse("""
    module  {
      func.func @main(%arg0: memref<*xcomplex<f32>>,
                      %arg1: memref<*xcomplex<f32>>,
                      %arg2: memref<*xcomplex<f32>>) attributes { llvm.emit_c_interface } {
        %A = memref.cast %arg0 : memref<*xcomplex<f32>> to memref<1xcomplex<f32>>
        %B = memref.cast %arg1 : memref<*xcomplex<f32>> to memref<1xcomplex<f32>>
        %C = memref.cast %arg2 : memref<*xcomplex<f32>> to memref<1xcomplex<f32>>
        %0 = arith.constant 0 : index
        %1 = memref.load %A[%0] : memref<1xcomplex<f32>>
        %2 = memref.load %B[%0] : memref<1xcomplex<f32>>
        %3 = complex.add %1, %2 : complex<f32>
        memref.store %3, %C[%0] : memref<1xcomplex<f32>>
        return
      }
    } """)

    arg1 = np.array([5.+6.j]).astype(np.complex64)
    arg2 = np.array([7.+8.j]).astype(np.complex64)
    arg3  = np.array([0.+0.j]).astype(np.complex64)

    arg1_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_unranked_memref_descriptor(arg1)))
    arg2_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_unranked_memref_descriptor(arg2)))
    arg3_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_unranked_memref_descriptor(arg3)))

    execution_engine = ExecutionEngine(lowerToLLVM(module))
    execution_engine.invoke("main",
                            arg1_memref_ptr,
                            arg2_memref_ptr,
                            arg3_memref_ptr)
    # CHECK: [5.+6.j] + [7.+8.j] = [12.+14.j]
    log("{0} + {1} = {2}".format(arg1, arg2, arg3))

    # test to-numpy utility
    # CHECK: [12.+14.j]
    npout = unranked_memref_to_numpy(arg3_memref_ptr[0],
                                     np.dtype(np.complex64))
    log(npout)


run(testComplexUnrankedMemrefAdd)


#  Test addition of two 2d_memref
# CHECK-LABEL: TEST: testDynamicMemrefAdd2D
def testDynamicMemrefAdd2D():
  with Context():
    module = Module.parse("""
      module  {
        func.func @memref_add_2d(%arg0: memref<2x2xf32>, %arg1: memref<?x?xf32>, %arg2: memref<2x2xf32>) attributes {llvm.emit_c_interface} {
          %c0 = arith.constant 0 : index
          %c2 = arith.constant 2 : index
          %c1 = arith.constant 1 : index
          cf.br ^bb1(%c0 : index)
        ^bb1(%0: index):  // 2 preds: ^bb0, ^bb5
          %1 = arith.cmpi slt, %0, %c2 : index
          cf.cond_br %1, ^bb2, ^bb6
        ^bb2:  // pred: ^bb1
          %c0_0 = arith.constant 0 : index
          %c2_1 = arith.constant 2 : index
          %c1_2 = arith.constant 1 : index
          cf.br ^bb3(%c0_0 : index)
        ^bb3(%2: index):  // 2 preds: ^bb2, ^bb4
          %3 = arith.cmpi slt, %2, %c2_1 : index
          cf.cond_br %3, ^bb4, ^bb5
        ^bb4:  // pred: ^bb3
          %4 = memref.load %arg0[%0, %2] : memref<2x2xf32>
          %5 = memref.load %arg1[%0, %2] : memref<?x?xf32>
          %6 = arith.addf %4, %5 : f32
          memref.store %6, %arg2[%0, %2] : memref<2x2xf32>
          %7 = arith.addi %2, %c1_2 : index
          cf.br ^bb3(%7 : index)
        ^bb5:  // pred: ^bb3
          %8 = arith.addi %0, %c1 : index
          cf.br ^bb1(%8 : index)
        ^bb6:  // pred: ^bb1
          return
        }
      }
        """)
    arg1 = np.random.randn(2, 2).astype(np.float32)
    arg2 = np.random.randn(2, 2).astype(np.float32)
    res = np.random.randn(2, 2).astype(np.float32)

    arg1_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg1)))
    arg2_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg2)))
    res_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(res)))

    execution_engine = ExecutionEngine(lowerToLLVM(module))
    execution_engine.invoke("memref_add_2d", arg1_memref_ptr, arg2_memref_ptr,
                            res_memref_ptr)
    # CHECK: True
    log(np.allclose(arg1 + arg2, res))


run(testDynamicMemrefAdd2D)


#  Test loading of shared libraries.
# CHECK-LABEL: TEST: testSharedLibLoad
def testSharedLibLoad():
  with Context():
    module = Module.parse("""
      module  {
      func.func @main(%arg0: memref<1xf32>) attributes { llvm.emit_c_interface } {
        %c0 = arith.constant 0 : index
        %cst42 = arith.constant 42.0 : f32
        memref.store %cst42, %arg0[%c0] : memref<1xf32>
        %u_memref = memref.cast %arg0 : memref<1xf32> to memref<*xf32>
        call @printMemrefF32(%u_memref) : (memref<*xf32>) -> ()
        return
      }
      func.func private @printMemrefF32(memref<*xf32>) attributes { llvm.emit_c_interface }
     } """)
    arg0 = np.array([0.0]).astype(np.float32)

    arg0_memref_ptr = ctypes.pointer(
        ctypes.pointer(get_ranked_memref_descriptor(arg0)))

    if sys.platform == 'win32':
      shared_libs = [
          "../../../../bin/mlir_runner_utils.dll",
          "../../../../bin/mlir_c_runner_utils.dll"
      ]
    elif sys.platform == 'darwin':
      shared_libs = [
          "../../../../lib/libmlir_runner_utils.dylib",
          "../../../../lib/libmlir_c_runner_utils.dylib"
      ]
    else:
      shared_libs = [
          "../../../../lib/libmlir_runner_utils.so",
          "../../../../lib/libmlir_c_runner_utils.so"
      ]

    execution_engine = ExecutionEngine(
        lowerToLLVM(module),
        opt_level=3,
        shared_libs=shared_libs)
    execution_engine.invoke("main", arg0_memref_ptr)
    # CHECK: Unranked Memref
    # CHECK-NEXT: [42]


run(testSharedLibLoad)


#  Test that nano time clock is available.
# CHECK-LABEL: TEST: testNanoTime
def testNanoTime():
  with Context():
    module = Module.parse("""
      module {
      func.func @main() attributes { llvm.emit_c_interface } {
        %now = call @nanoTime() : () -> i64
        %memref = memref.alloca() : memref<1xi64>
        %c0 = arith.constant 0 : index
        memref.store %now, %memref[%c0] : memref<1xi64>
        %u_memref = memref.cast %memref : memref<1xi64> to memref<*xi64>
        call @printMemrefI64(%u_memref) : (memref<*xi64>) -> ()
        return
      }
      func.func private @nanoTime() -> i64 attributes { llvm.emit_c_interface }
      func.func private @printMemrefI64(memref<*xi64>) attributes { llvm.emit_c_interface }
    }""")

    if sys.platform == 'win32':
      shared_libs = [
          "../../../../bin/mlir_runner_utils.dll",
          "../../../../bin/mlir_c_runner_utils.dll"
      ]
    else:
      shared_libs = [
          "../../../../lib/libmlir_runner_utils.so",
          "../../../../lib/libmlir_c_runner_utils.so"
      ]

    execution_engine = ExecutionEngine(
        lowerToLLVM(module),
        opt_level=3,
        shared_libs=shared_libs)
    execution_engine.invoke("main")
    # CHECK: Unranked Memref
    # CHECK: [{{.*}}]


run(testNanoTime)



import lldb


class jasSynthProvider:

    def __init__(self, valobj, dict):
        self.valobj = valobj

    def num_children(self):
        return 2

    def get_child_at_index(self, index):
        child = None
        if index == 0:
            child = self.valobj.GetChildMemberWithName('A')
        if index == 1:
            child = self.valobj.CreateValueFromExpression('X', '(int)1')
        return child

    def get_child_index(self, name):
        if name == 'A':
            return 0
        if name == 'X':
            return 1
        return None


def ccc_summary(sbvalue, internal_dict):
    sbvalue = sbvalue.GetNonSyntheticValue()
    # This tests that the SBValue.GetNonSyntheticValue() actually returns a
    # non-synthetic value. If it does not, then sbvalue.GetChildMemberWithName("a")
    # in the following statement will call the 'get_child_index' method of the
    # synthetic child provider CCCSynthProvider below (which raises an
    # exception).
    return "CCC object with leading value " + \
        str(sbvalue.GetChildMemberWithName("a"))


class CCCSynthProvider(object):

    def __init__(self, sbvalue, internal_dict):
        self._sbvalue = sbvalue

    def num_children(self):
        return 3

    def get_child_index(self, name):
        raise RuntimeError("I don't want to be called!")

    def get_child_at_index(self, index):
        if index == 0:
            return self._sbvalue.GetChildMemberWithName("a")
        if index == 1:
            return self._sbvalue.GetChildMemberWithName("b")
        if index == 2:
            return self._sbvalue.GetChildMemberWithName("c")


def empty1_summary(sbvalue, internal_dict):
    return "I am an empty Empty1"


class Empty1SynthProvider(object):

    def __init__(self, sbvalue, internal_dict):
        self._sbvalue = sbvalue

    def num_children(self):
        return 0

    def get_child_at_index(self, index):
        return None


def empty2_summary(sbvalue, internal_dict):
    return "I am an empty Empty2"


class Empty2SynthProvider(object):

    def __init__(self, sbvalue, internal_dict):
        self._sbvalue = sbvalue

    def num_children(self):
        return 0

    def get_child_at_index(self, index):
        return None


def __lldb_init_module(debugger, dict):
    debugger.CreateCategory("JASSynth").AddTypeSynthetic(
        lldb.SBTypeNameSpecifier("JustAStruct"),
        lldb.SBTypeSynthetic.CreateWithClassName("synth.jasSynthProvider"))
    cat = debugger.CreateCategory("CCCSynth")
    cat.AddTypeSynthetic(
        lldb.SBTypeNameSpecifier("CCC"),
        lldb.SBTypeSynthetic.CreateWithClassName("synth.CCCSynthProvider",
                                                 lldb.eTypeOptionCascade))
    cat.AddTypeSummary(
        lldb.SBTypeNameSpecifier("CCC"),
        lldb.SBTypeSummary.CreateWithFunctionName("synth.ccc_summary",
                                                  lldb.eTypeOptionCascade))
    cat.AddTypeSynthetic(
        lldb.SBTypeNameSpecifier("Empty1"),
        lldb.SBTypeSynthetic.CreateWithClassName("synth.Empty1SynthProvider"))
    cat.AddTypeSummary(
        lldb.SBTypeNameSpecifier("Empty1"),
        lldb.SBTypeSummary.CreateWithFunctionName("synth.empty1_summary"))
    cat.AddTypeSynthetic(
        lldb.SBTypeNameSpecifier("Empty2"),
        lldb.SBTypeSynthetic.CreateWithClassName("synth.Empty2SynthProvider"))
    cat.AddTypeSummary(
        lldb.SBTypeNameSpecifier("Empty2"),
        lldb.SBTypeSummary.CreateWithFunctionName(
            "synth.empty2_summary",
            lldb.eTypeOptionHideEmptyAggregates))



#!/usr/bin/env python
#===- test.py -  ---------------------------------------------*- python -*--===#
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
#===------------------------------------------------------------------------===#

from cppreference_parser import _ParseSymbolPage, _ParseIndexPage

import unittest

class TestStdGen(unittest.TestCase):

  def testParseIndexPage(self):
    html = """
 <a href="abs.html" title="abs"><tt>abs()</tt></a> (int) <br>
 <a href="complex/abs.html" title="abs"><tt>abs&lt;&gt;()</tt></a> (std::complex) <br>
 <a href="acos.html" title="acos"><tt>acos()</tt></a> <br>
 <a href="acosh.html" title="acosh"><tt>acosh()</tt></a> <span class="t-mark-rev">(since C++11)</span> <br>
 <a href="as_bytes.html" title="as bytes"><tt>as_bytes&lt;&gt;()</tt></a> <span class="t-mark-rev t-since-cxx20">(since C++20)</span> <br>
 """

    actual = _ParseIndexPage(html)
    expected = [
      ("abs", "abs.html", True),
      ("abs", "complex/abs.html", True),
      ("acos", "acos.html", False),
      ("acosh", "acosh.html", False),
      ("as_bytes", "as_bytes.html", False),
    ]
    self.assertEqual(len(actual), len(expected))
    for i in range(0, len(actual)):
      self.assertEqual(expected[i][0], actual[i][0])
      self.assertTrue(actual[i][1].endswith(expected[i][1]))
      self.assertEqual(expected[i][2], actual[i][2])


  def testParseSymbolPage_SingleHeader(self):
    # Defined in header <cmath>
    html = """
 <table class="t-dcl-begin"><tbody>
  <tr class="t-dsc-header">
  <td> <div>Defined in header <code><a href="cmath.html" title="cmath">&lt;cmath&gt;</a></code>
   </div></td>
  <td></td>
  <td></td>
  </tr>
  <tr class="t-dcl">
    <td>void foo()</td>
    <td>this is matched</td>
  </tr>
</tbody></table>
"""
    self.assertEqual(_ParseSymbolPage(html, 'foo'), set(['<cmath>']))


  def testParseSymbolPage_MulHeaders(self):
    #  Defined in header <cstddef>
    #  Defined in header <cstdio>
    #  Defined in header <cstdlib>
    html = """
<table class="t-dcl-begin"><tbody>
  <tr class="t-dsc-header">
    <td> <div>Defined in header <code><a href="cstddef.html" title="cstddef">&lt;cstddef&gt;</a></code>
     </div></td>
     <td></td>
    <td></td>
  </tr>
  <tr class="t-dcl">
    <td>void bar()</td>
    <td>this mentions foo, but isn't matched</td>
  </tr>
  <tr class="t-dsc-header">
    <td> <div>Defined in header <code><a href="cstdio.html" title="cstdio">&lt;cstdio&gt;</a></code>
     </div></td>
    <td></td>
    <td></td>
  </tr>
  <tr class="t-dsc-header">
    <td> <div>Defined in header <code><a href=".cstdlib.html" title="ccstdlib">&lt;cstdlib&gt;</a></code>
     </div></td>
    <td></td>
    <td></td>
  </tr>
  <tr class="t-dcl">
    <td>
      <span>void</span>
      foo
      <span>()</span>
    </td>
    <td>this is matched</td>
  </tr>
</tbody></table>
"""
    self.assertEqual(_ParseSymbolPage(html, "foo"),
                     set(['<cstdio>', '<cstdlib>']))


  def testParseSymbolPage_MulHeadersInSameDiv(self):
    # Multile <code> blocks in a Div.
    # Defined in header <algorithm>
    # Defined in header <utility>
    html = """
<table class="t-dcl-begin"><tbody>
<tr class="t-dsc-header">
<td><div>
     Defined in header <code><a href="../header/algorithm.html" title="cpp/header/algorithm">&lt;algorithm&gt;</a></code><br>
     Defined in header <code><a href="../header/utility.html" title="cpp/header/utility">&lt;utility&gt;</a></code>
</div></td>
<td></td>
</tr>
<tr class="t-dcl">
  <td>
    <span>void</span>
    foo
    <span>()</span>
  </td>
  <td>this is matched</td>
</tr>
</tbody></table>
"""
    self.assertEqual(_ParseSymbolPage(html, "foo"),
                     set(['<algorithm>', '<utility>']))

  def testParseSymbolPage_MulSymbolsInSameTd(self):
    # defined in header <cstdint>
    #   int8_t
    #   int16_t
    html = """
<table class="t-dcl-begin"><tbody>
<tr class="t-dsc-header">
<td><div>
     Defined in header <code><a href="cstdint.html" title="cstdint">&lt;cstdint&gt;</a></code><br>
</div></td>
<td></td>
</tr>
<tr class="t-dcl">
  <td>
    <span>int8_t</span>
    <span>int16_t</span>
  </td>
  <td>this is matched</td>
</tr>
</tbody></table>
"""
    self.assertEqual(_ParseSymbolPage(html, "int8_t"),
                     set(['<cstdint>']))
    self.assertEqual(_ParseSymbolPage(html, "int16_t"),
                     set(['<cstdint>']))


if __name__ == '__main__':
  unittest.main()



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class ExprXValuePrintingTestCase(TestBase):

    @expectedFailureAll(oslist=["windows"], archs=["i[3-6]86", "x86_64"], bugnumber="llvm.org/pr21765")
    def test(self):
        """Printing an xvalue should work."""
        self.build()
        lldbutil.run_to_source_breakpoint(self, '// Break here', lldb.SBFileSpec("main.cpp"))
        self.expect_expr("foo().data", result_value="1234")



"""
Test the diagnostics emitted by our embeded Clang instance that parses expressions.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *

class ExprDiagnosticsTestCase(TestBase):

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        self.main_source = "main.cpp"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def test_source_and_caret_printing(self):
        """Test that the source and caret positions LLDB prints are correct"""
        self.build()

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(self,
                                          '// Break here', self.main_source_spec)
        frame = thread.GetFrameAtIndex(0)

        # Test that source/caret are at the right position.
        value = frame.EvaluateExpression("unknown_identifier")
        self.assertFalse(value.GetError().Success())
        # We should get a nice diagnostic with a caret pointing at the start of
        # the identifier.
        self.assertIn("\nunknown_identifier\n^\n", value.GetError().GetCString())
        self.assertIn("<user expression 0>:1:1", value.GetError().GetCString())

        # Same as above but with the identifier in the middle.
        value = frame.EvaluateExpression("1 + unknown_identifier  ")
        self.assertFalse(value.GetError().Success())
        self.assertIn("\n1 + unknown_identifier", value.GetError().GetCString())
        self.assertIn("\n    ^\n", value.GetError().GetCString())

        # Multiline expressions.
        value = frame.EvaluateExpression("int a = 0;\nfoobar +=1;\na")
        self.assertFalse(value.GetError().Success())
        # We should still get the right line information and caret position.
        self.assertIn("\nfoobar +=1;\n^\n", value.GetError().GetCString())
        # It's the second line of the user expression.
        self.assertIn("<user expression 2>:2:1", value.GetError().GetCString())

        # Top-level expressions.
        top_level_opts = lldb.SBExpressionOptions();
        top_level_opts.SetTopLevel(True)

        value = frame.EvaluateExpression("void foo(unknown_type x) {}", top_level_opts)
        self.assertFalse(value.GetError().Success())
        self.assertIn("\nvoid foo(unknown_type x) {}\n         ^\n", value.GetError().GetCString())
        # Top-level expressions might use a different wrapper code, but the file name should still
        # be the same.
        self.assertIn("<user expression 3>:1:10", value.GetError().GetCString())

        # Multiline top-level expressions.
        value = frame.EvaluateExpression("void x() {}\nvoid foo;", top_level_opts)
        self.assertFalse(value.GetError().Success())
        self.assertIn("\nvoid foo;\n     ^", value.GetError().GetCString())
        self.assertIn("<user expression 4>:2:6", value.GetError().GetCString())

        # Test that we render Clang's 'notes' correctly.
        value = frame.EvaluateExpression("struct SFoo{}; struct SFoo { int x; };", top_level_opts)
        self.assertFalse(value.GetError().Success())
        self.assertIn("<user expression 5>:1:8: previous definition is here\nstruct SFoo{}; struct SFoo { int x; };\n       ^\n", value.GetError().GetCString())

        # Declarations from the debug information currently have no debug information. It's not clear what
        # we should do in this case, but we should at least not print anything that's wrong.
        # In the future our declarations should have valid source locations.
        value = frame.EvaluateExpression("struct FooBar { double x };", top_level_opts)
        self.assertFalse(value.GetError().Success())
        self.assertIn("error: <user expression 6>:1:8: redefinition of 'FooBar'\nstruct FooBar { double x };\n       ^\n", value.GetError().GetCString())

        value = frame.EvaluateExpression("foo(1, 2)")
        self.assertFalse(value.GetError().Success())
        self.assertIn("error: <user expression 7>:1:1: no matching function for call to 'foo'\nfoo(1, 2)\n^~~\nnote: candidate function not viable: requires single argument 'x', but 2 arguments were provided\n\n", value.GetError().GetCString())

        # Redefine something that we defined in a user-expression. We should use the previous expression file name
        # for the original decl.
        value = frame.EvaluateExpression("struct Redef { double x; };", top_level_opts)
        value = frame.EvaluateExpression("struct Redef { float y; };", top_level_opts)
        self.assertFalse(value.GetError().Success())
        self.assertIn("error: <user expression 9>:1:8: redefinition of 'Redef'\nstruct Redef { float y; };\n       ^\n<user expression 8>:1:8: previous definition is here\nstruct Redef { double x; };\n       ^", value.GetError().GetCString())

    @add_test_categories(["objc"])
    def test_source_locations_from_objc_modules(self):
        self.build()

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(self,
                                          '// Break here', self.main_source_spec)
        frame = thread.GetFrameAtIndex(0)

        # Import foundation so that the Obj-C module is loaded (which contains source locations
        # that can be used by LLDB).
        self.runCmd("expr @import Foundation")
        value = frame.EvaluateExpression("NSLog(1);")
        self.assertFalse(value.GetError().Success())
        # LLDB should print the source line that defines NSLog. To not rely on any
        # header paths/line numbers or the actual formatting of the Foundation headers, only look
        # for a few tokens in the output.
        # File path should come from Foundation framework.
        self.assertIn("/Foundation.framework/", value.GetError().GetCString())
        # The NSLog definition source line should be printed. Return value and
        # the first argument are probably stable enough that this test can check for them.
        self.assertIn("void NSLog(NSString *format", value.GetError().GetCString())




# Usage:
#   art/test/run-test --host --gdb [--64] [--interpreter] 004-JniTest
#   'b Java_Main_shortMethod'
#   'r'
#   'command script import host_art_bt.py'
#   'host_art_bt'

from __future__ import print_function

import sys
import re

import lldb


def host_art_bt(debugger, command, result, internal_dict):
    prettified_frames = []
    lldb_frame_index = 0
    art_frame_index = 0
    target = debugger.GetSelectedTarget()
    process = target.GetProcess()
    thread = process.GetSelectedThread()
    while lldb_frame_index < thread.GetNumFrames():
        frame = thread.GetFrameAtIndex(lldb_frame_index)
        if frame.GetModule() and re.match(r'JIT\(.*?\)',
                                          frame.GetModule().GetFileSpec().GetFilename()):
            # Compiled Java frame

            # Get function/filename/lineno from symbol context
            symbol = frame.GetSymbol()
            if not symbol:
                print('No symbol info for compiled Java frame: ', frame)
                sys.exit(1)
            line_entry = frame.GetLineEntry()
            prettified_frames.append({
                'function': symbol.GetName(),
                'file': str(line_entry.GetFileSpec()) if line_entry else None,
                'line': line_entry.GetLine() if line_entry else -1
            })

            # Skip art frames
            while True:
                art_stack_visitor = frame.EvaluateExpression(
                    """struct GetStackVisitor : public StackVisitor { GetStackVisitor(int depth_) : StackVisitor(Thread::Current(), NULL), depth(depth_) {} bool VisitFrame() { if (cur_depth_ == depth) { return false; } else { return true; } } int depth; }; GetStackVisitor visitor(""" +
                    str(art_frame_index) +
                    """); visitor.WalkStack(true); visitor""")
                art_method = frame.EvaluateExpression(
                    art_stack_visitor.GetName() + """.GetMethod()""")
                if art_method.GetValueAsUnsigned() != 0:
                    art_method_name = frame.EvaluateExpression(
                        """art::PrettyMethod(""" + art_method.GetName() + """, true)""")
                    art_method_name_data = frame.EvaluateExpression(
                        art_method_name.GetName() + """.c_str()""").GetValueAsUnsigned()
                    art_method_name_size = frame.EvaluateExpression(
                        art_method_name.GetName() + """.length()""").GetValueAsUnsigned()
                    error = lldb.SBError()
                    art_method_name = process.ReadCStringFromMemory(
                        art_method_name_data, art_method_name_size + 1, error)
                    if not error.Success:
                        print('Failed to read method name')
                        sys.exit(1)
                    if art_method_name != symbol.GetName():
                        print('Function names in native symbol and art runtime stack do not match: ', symbol.GetName(), ' != ', art_method_name)
                    art_frame_index = art_frame_index + 1
                    break
                art_frame_index = art_frame_index + 1

            # Skip native frames
            lldb_frame_index = lldb_frame_index + 1
            if lldb_frame_index < thread.GetNumFrames():
                frame = thread.GetFrameAtIndex(lldb_frame_index)
                if frame.GetModule() and re.match(
                        r'JIT\(.*?\)', frame.GetModule().GetFileSpec().GetFilename()):
                    # Another compile Java frame
                    # Don't skip; leave it to the next iteration
                    continue
                elif frame.GetSymbol() and (frame.GetSymbol().GetName() == 'art_quick_invoke_stub' or frame.GetSymbol().GetName() == 'art_quick_invoke_static_stub'):
                    # art_quick_invoke_stub / art_quick_invoke_static_stub
                    # Skip until we get past the next ArtMethod::Invoke()
                    while True:
                        lldb_frame_index = lldb_frame_index + 1
                        if lldb_frame_index >= thread.GetNumFrames():
                            print('ArtMethod::Invoke not found below art_quick_invoke_stub/art_quick_invoke_static_stub')
                            sys.exit(1)
                        frame = thread.GetFrameAtIndex(lldb_frame_index)
                        if frame.GetSymbol() and frame.GetSymbol().GetName(
                        ) == 'art::mirror::ArtMethod::Invoke(art::Thread*, unsigned int*, unsigned int, art::JValue*, char const*)':
                            lldb_frame_index = lldb_frame_index + 1
                            break
                else:
                    print('Invalid frame below compiled Java frame: ', frame)
        elif frame.GetSymbol() and frame.GetSymbol().GetName() == 'art_quick_generic_jni_trampoline':
            # Interpreted JNI frame for x86_64

            # Skip art frames
            while True:
                art_stack_visitor = frame.EvaluateExpression(
                    """struct GetStackVisitor : public StackVisitor { GetStackVisitor(int depth_) : StackVisitor(Thread::Current(), NULL), depth(depth_) {} bool VisitFrame() { if (cur_depth_ == depth) { return false; } else { return true; } } int depth; }; GetStackVisitor visitor(""" +
                    str(art_frame_index) +
                    """); visitor.WalkStack(true); visitor""")
                art_method = frame.EvaluateExpression(
                    art_stack_visitor.GetName() + """.GetMethod()""")
                if art_method.GetValueAsUnsigned() != 0:
                    # Get function/filename/lineno from ART runtime
                    art_method_name = frame.EvaluateExpression(
                        """art::PrettyMethod(""" + art_method.GetName() + """, true)""")
                    art_method_name_data = frame.EvaluateExpression(
                        art_method_name.GetName() + """.c_str()""").GetValueAsUnsigned()
                    art_method_name_size = frame.EvaluateExpression(
                        art_method_name.GetName() + """.length()""").GetValueAsUnsigned()
                    error = lldb.SBError()
                    function = process.ReadCStringFromMemory(
                        art_method_name_data, art_method_name_size + 1, error)

                    prettified_frames.append({
                        'function': function,
                        'file': None,
                        'line': -1
                    })

                    art_frame_index = art_frame_index + 1
                    break
                art_frame_index = art_frame_index + 1

            # Skip native frames
            lldb_frame_index = lldb_frame_index + 1
            if lldb_frame_index < thread.GetNumFrames():
                frame = thread.GetFrameAtIndex(lldb_frame_index)
                if frame.GetSymbol() and (frame.GetSymbol().GetName() ==
                                          'art_quick_invoke_stub' or frame.GetSymbol().GetName() == 'art_quick_invoke_static_stub'):
                    # art_quick_invoke_stub / art_quick_invoke_static_stub
                    # Skip until we get past the next ArtMethod::Invoke()
                    while True:
                        lldb_frame_index = lldb_frame_index + 1
                        if lldb_frame_index >= thread.GetNumFrames():
                            print('ArtMethod::Invoke not found below art_quick_invoke_stub/art_quick_invoke_static_stub')
                            sys.exit(1)
                        frame = thread.GetFrameAtIndex(lldb_frame_index)
                        if frame.GetSymbol() and frame.GetSymbol().GetName(
                        ) == 'art::mirror::ArtMethod::Invoke(art::Thread*, unsigned int*, unsigned int, art::JValue*, char const*)':
                            lldb_frame_index = lldb_frame_index + 1
                            break
                else:
                    print('Invalid frame below compiled Java frame: ', frame)
        elif frame.GetSymbol() and re.search(r'art::interpreter::', frame.GetSymbol().GetName()):
            # Interpreted Java frame

            while True:
                lldb_frame_index = lldb_frame_index + 1
                if lldb_frame_index >= thread.GetNumFrames():
                    print('art::interpreter::Execute not found in interpreter frame')
                    sys.exit(1)
                frame = thread.GetFrameAtIndex(lldb_frame_index)
                if frame.GetSymbol() and frame.GetSymbol().GetName(
                ) == 'art::interpreter::Execute(art::Thread*, art::MethodHelper&, art::DexFile::CodeItem const*, art::ShadowFrame&, art::JValue)':
                    break

            # Skip art frames
            while True:
                art_stack_visitor = frame.EvaluateExpression(
                    """struct GetStackVisitor : public StackVisitor { GetStackVisitor(int depth_) : StackVisitor(Thread::Current(), NULL), depth(depth_) {} bool VisitFrame() { if (cur_depth_ == depth) { return false; } else { return true; } } int depth; }; GetStackVisitor visitor(""" +
                    str(art_frame_index) +
                    """); visitor.WalkStack(true); visitor""")
                art_method = frame.EvaluateExpression(
                    art_stack_visitor.GetName() + """.GetMethod()""")
                if art_method.GetValueAsUnsigned() != 0:
                    # Get function/filename/lineno from ART runtime
                    art_method_name = frame.EvaluateExpression(
                        """art::PrettyMethod(""" + art_method.GetName() + """, true)""")
                    art_method_name_data = frame.EvaluateExpression(
                        art_method_name.GetName() + """.c_str()""").GetValueAsUnsigned()
                    art_method_name_size = frame.EvaluateExpression(
                        art_method_name.GetName() + """.length()""").GetValueAsUnsigned()
                    error = lldb.SBError()
                    function = process.ReadCStringFromMemory(
                        art_method_name_data, art_method_name_size + 1, error)

                    line = frame.EvaluateExpression(
                        art_stack_visitor.GetName() +
                        """.GetMethod()->GetLineNumFromDexPC(""" +
                        art_stack_visitor.GetName() +
                        """.GetDexPc(true))""").GetValueAsUnsigned()

                    file_name = frame.EvaluateExpression(
                        art_method.GetName() + """->GetDeclaringClassSourceFile()""")
                    file_name_data = file_name.GetValueAsUnsigned()
                    file_name_size = frame.EvaluateExpression(
                        """(size_t)strlen(""" + file_name.GetName() + """)""").GetValueAsUnsigned()
                    error = lldb.SBError()
                    file_name = process.ReadCStringFromMemory(
                        file_name_data, file_name_size + 1, error)
                    if not error.Success():
                        print('Failed to read source file name')
                        sys.exit(1)

                    prettified_frames.append({
                        'function': function,
                        'file': file_name,
                        'line': line
                    })

                    art_frame_index = art_frame_index + 1
                    break
                art_frame_index = art_frame_index + 1

            # Skip native frames
            while True:
                lldb_frame_index = lldb_frame_index + 1
                if lldb_frame_index >= thread.GetNumFrames():
                    print('Can not get past interpreter native frames')
                    sys.exit(1)
                frame = thread.GetFrameAtIndex(lldb_frame_index)
                if frame.GetSymbol() and not re.search(
                        r'art::interpreter::', frame.GetSymbol().GetName()):
                    break
        else:
            # Other frames. Add them as-is.
            frame = thread.GetFrameAtIndex(lldb_frame_index)
            lldb_frame_index = lldb_frame_index + 1
            if frame.GetModule():
                module_name = frame.GetModule().GetFileSpec().GetFilename()
                if not module_name in [
                    'libartd.so',
                    'dalvikvm32',
                    'dalvikvm64',
                        'libc.so.6']:
                    prettified_frames.append({
                        'function': frame.GetSymbol().GetName() if frame.GetSymbol() else None,
                        'file': str(frame.GetLineEntry().GetFileSpec()) if frame.GetLineEntry() else None,
                        'line': frame.GetLineEntry().GetLine() if frame.GetLineEntry() else -1
                    })

    for prettified_frame in prettified_frames:
        print(prettified_frame['function'], prettified_frame['file'], prettified_frame['line'])


def __lldb_init_module(debugger, internal_dict):
    debugger.HandleCommand(
        'command script add -f host_art_bt.host_art_bt host_art_bt')



"""
This tests some simple examples of parsing regex commands
"""

import os
import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestCommandRegexParsing(TestBase):

    NO_DEBUG_INFO_TESTCASE = True

    def test_sample_rename_this(self):
        """Try out some simple regex commands, make sure they parse correctly."""
        self.runCmd("command regex one-substitution 's/(.+)/echo-cmd %1-first %1-second %1-third/'")
        self.expect("one-substitution ASTRING",
                    substrs = ["ASTRING-first", "ASTRING-second", "ASTRING-third"])

        self.runCmd("command regex two-substitution 's/([^ ]+) ([^ ]+)/echo-cmd %1-first %2-second %1-third %2-fourth/'")
        self.expect("two-substitution ASTRING BSTRING",
                    substrs = ["ASTRING-first", "BSTRING-second", "ASTRING-third", "BSTRING-fourth"])
        
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.runCmd("command script import " + os.path.join(self.getSourceDir(), "echo_command.py"))
        self.runCmd("command script add echo-cmd -f echo_command.echo_command")



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""Provides POSIX implementation of formatted/colored console output."""

from ..PrettyOutputBase import PrettyOutputBase, _lock


class PrettyOutput(PrettyOutputBase):
    def _color(self, text, color, stream, lock=_lock):
        """Use ANSI escape codes to provide color on Linux."""
        stream = self._set_valid_stream(stream)
        with lock:
            if stream.color_enabled:
                text = '\033[{}m{}\033[0m'.format(color, text)
            self._write(text, stream)

    def red_impl(self, text, stream=None, **kwargs):
        self._color(text, 91, stream, **kwargs)

    def yellow_impl(self, text, stream=None, **kwargs):
        self._color(text, 93, stream, **kwargs)

    def green_impl(self, text, stream=None, **kwargs):
        self._color(text, 92, stream, **kwargs)

    def blue_impl(self, text, stream=None, **kwargs):
        self._color(text, 96, stream, **kwargs)

    def default_impl(self, text, stream=None, **kwargs):
        self._color(text, 0, stream, **kwargs)
